// BaseMesh.cpp
// Base mesh class, for inheriting base mesh functionality.

#include "basemesh.h"

BaseMesh::BaseMesh()
{
}


BaseMesh::~BaseMesh()
{
	// Release the index buffer.
	if (m_indexBuffer)
	{
		m_indexBuffer->Release();
		m_indexBuffer = 0;
	}

	// Release the vertex buffer.
	if (m_vertexBuffer)
	{
		m_vertexBuffer->Release();
		m_vertexBuffer = 0;
	}

	// Release the texture object.
	if (m_Texture)
	{
		delete m_Texture;
		m_Texture = 0;
	}
}

int BaseMesh::GetIndexCount()
{
	return m_indexCount;
}

ID3D11ShaderResourceView* BaseMesh::GetTexture()
{
	return m_Texture->GetTexture();
}

ID3D11ShaderResourceView* BaseMesh::GetBumpMap()
{
	return m_BumpMap->GetTexture();
}

void BaseMesh::SendData(ID3D11DeviceContext* deviceContext)
{
	unsigned int stride;
	unsigned int offset;
	
	// Set vertex buffer stride and offset.
	stride = (m_boneAnimated) ? sizeof(AnimVertexType) : sizeof(VertexType);
	offset = 0;

	// Set the vertex buffer to active in the input assembler so it can be rendered.
	deviceContext->IASetVertexBuffers(0, 1, &m_vertexBuffer, &stride, &offset);

	// Set the index buffer to active in the input assembler so it can be rendered.
	deviceContext->IASetIndexBuffer(m_indexBuffer, DXGI_FORMAT_R32_UINT, 0);

	// Set the type of primitive that should be rendered from this vertex buffer, in this case triangles.
	deviceContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}

void BaseMesh::LoadTexture(ID3D11Device* device, ID3D11DeviceContext* deviceContext, WCHAR* filename)
{
	// Create the texture object.
	m_Texture = new Texture(device, deviceContext, filename);
}

void BaseMesh::LoadBumpMap(ID3D11Device* device, ID3D11DeviceContext* deviceContext, WCHAR* filename)
{
	m_BumpMap = new Texture(device, deviceContext, filename);
}

void BaseMesh::CalculateModelVectors(VertexType* vertices)
{
	int faceCount, index;
	TempVertexType vertex1, vertex2, vertex3;
	VectorType tangent;

	faceCount = m_vertexCount / 3;

	index = 0;

	for (int i = 0; i < faceCount; i++)
	{
		// Get the three vertices for this face from the model.
		vertex1.x = vertices[index].position.x;
		vertex1.y = vertices[index].position.y;
		vertex1.z = vertices[index].position.z;
		vertex1.tu = vertices[index].texture.x;
		vertex1.tv = vertices[index].texture.y;
		vertex1.nx = vertices[index].normal.x;
		vertex1.ny = vertices[index].normal.y;
		vertex1.nz = vertices[index].normal.z;
		index++;

		vertex2.x = vertices[index].position.x;
		vertex2.y = vertices[index].position.y;
		vertex2.z = vertices[index].position.z;
		vertex2.tu = vertices[index].texture.x;
		vertex2.tv = vertices[index].texture.y;
		vertex2.nx = vertices[index].normal.x;
		vertex2.ny = vertices[index].normal.y;
		vertex2.nz = vertices[index].normal.z;
		index++;

		vertex3.x = vertices[index].position.x;
		vertex3.y = vertices[index].position.y;
		vertex3.z = vertices[index].position.z;
		vertex3.tu = vertices[index].texture.x;
		vertex3.tv = vertices[index].texture.y;
		vertex3.nx = vertices[index].normal.x;
		vertex3.ny = vertices[index].normal.y;
		vertex3.nz = vertices[index].normal.z;
		index++;

		// Calculate the tangent and binormal of that face.
		CalculateTangent(vertex1, vertex2, vertex3, tangent);

		vertices[index - 1].tangent.x = tangent.x;
		vertices[index - 1].tangent.y = tangent.y;
		vertices[index - 1].tangent.z = tangent.z;

		vertices[index - 2].tangent.x = tangent.x;
		vertices[index - 2].tangent.y = tangent.y;
		vertices[index - 2].tangent.z = tangent.z;

		vertices[index - 3].tangent.x = tangent.x;
		vertices[index - 3].tangent.y = tangent.y;
		vertices[index - 3].tangent.z = tangent.z;
	}
}

void BaseMesh::CalculateNormal(VectorType v1, VectorType v2, VectorType& normal)
{
}

void BaseMesh::CalculateTangent(TempVertexType v1, TempVertexType v2, TempVertexType v3, VectorType& tangent)
{
	XMFLOAT3 vector1, vector2;
	XMFLOAT2 tuVector, tvVector;
	float den;
	float length;

	vector1 = XMFLOAT3(v2.x - v1.x, v2.y - v1.y, v2.z - v1.z);
	vector2 = XMFLOAT3(v3.x - v1.x, v3.y - v1.y, v3.z - v1.z);

	tuVector = XMFLOAT2(v2.tu - v1.tu, v3.tu - v1.tu);
	tvVector = XMFLOAT2(v2.tv - v1.tv, v3.tv - v1.tv);

	den = 1.0f / (tuVector.x * tvVector.y - tuVector.y * tvVector.x);

	tangent.x = (tvVector.y * vector1.x - tvVector.x * vector2.x) * den;
	tangent.y = (tvVector.y * vector1.y - tvVector.x * vector2.y) * den;
	tangent.z = (tvVector.y * vector1.z - tvVector.x * vector2.z) * den;

	length = sqrt((tangent.x * tangent.x) + (tangent.y * tangent.y) + (tangent.z * tangent.z));

	tangent.x = tangent.x / length;
	tangent.y = tangent.y / length;
	tangent.z = tangent.z / length;
}

ID3D11Buffer* BaseMesh::GetVertexBuffer()
{
	return m_vertexBuffer;
}

RenderType BaseMesh::GetRenderType()
{
	return m_renderType;
}