// testure shader.h
#ifndef _BASESHADER_H_
#define _BASESHADER_H_

#include "D3D.h"
#include "Light.h"
#include "BaseMesh.h"
#include "SceneObject.h"

#include <d3d11.h>
#include <D3Dcompiler.h>
#include <dxgi.h>
#include <DirectXMath.h>
#include <fstream>

using namespace std;
using namespace DirectX;


class BaseShader
{
protected:
	struct MatrixBufferType
	{
		XMMATRIX world;
		XMMATRIX view;
		XMMATRIX projection;
	};

public:
	void* operator new(size_t i)
	{
		return _mm_malloc(i, 16);
	}

	void operator delete(void* p)
	{
		_mm_free(p);
	}

	BaseShader(ID3D11Device* device, HWND hwnd);
	~BaseShader();

	virtual void Render(ID3D11DeviceContext* deviceContext, int vertexCount);
	virtual void Render(ID3D11DeviceContext* deviceContext, D3D* direct3d, void* resourceContainer);

	void AddLight(int lightIndex);
	void RemoveLight(int lightIndex);

	void AddObject(SceneObject* object);
	void RemoveObject(int objectIndex);
	void RemoveObject(SceneObject* object);
	void RemoveObject(std::wstring& name);

	void AddCamera(int cameraIndex);
	void RemoveCamera(int cameraIndex);

	void AddBuffers(int bufferIndex);
	void RemoveBuffers(int bufferIndex);

	void AddUAV(int uavIndex);
	void RemoveUAV(int uavIndex);

	void AddTexture(int textureIndex);
	void RemoveTexture(int textureIndex);

	void AddFloat(int floatIndex);
	void RemoveFloat(int floatIndex);

	void SetLayout(ID3D11InputLayout* layout) { m_layout = layout; }
	void SetBlendState(ID3D11BlendState* blendState) { m_blendState = blendState; }
	void SetDepthStencilState(ID3D11DepthStencilState* depthStencilState) { m_depthStencilState = depthStencilState; }
	void SetRasterizerState(ID3D11RasterizerState* rasterizerState) { m_rasterizerState = rasterizerState; }

	virtual void ConfigurePipeline(ID3D11DeviceContext* deviceContext);

protected:
	virtual void InitShader(WCHAR*, WCHAR*) = 0;
	virtual void ClearPipeline(ID3D11DeviceContext* deviceContext);
	void OutputShaderErrorMessage(ID3D10Blob*, HWND, WCHAR*);
	void loadVertexShader(WCHAR* filename, LPCSTR entrypoint = "main");
	void loadHullShader(WCHAR* filename, LPCSTR entrypoint = "main");
	void loadDomainShader(WCHAR* filename, LPCSTR entrypoint = "main");
	void loadGeometryShader(WCHAR* filename, LPCSTR entrypoint = "main");
	void loadGeometryShaderWithStreamOutput(WCHAR* filename, D3D11_SO_DECLARATION_ENTRY* soDesc, int soDescCount, int rasterizedStream, LPCSTR entrypoint = "main");
	void loadPixelShader(WCHAR* filename, LPCSTR entrypoint = "main");
	void loadComputeShader(WCHAR* filename, LPCSTR entrypoint = "main");

protected:
	ID3D11Device* m_device;
	HWND m_hwnd;

	// Effect Shaders
	ID3D11VertexShader* m_vertexShader;
	ID3D11HullShader* m_hullShader;
	ID3D11DomainShader* m_domainShader;
	ID3D11GeometryShader* m_geometryShader;
	ID3D11PixelShader* m_pixelShader;
	ID3D11ComputeShader* m_computeShader;

	// Effect Parameters (Scene) - Allows for cleaner render passes CPU side
	std::vector<int> m_renderLights;
	std::vector<int> m_renderCameras;
	std::vector<int> m_renderBuffers;
	std::vector<int> m_renderUAVs;
	std::vector<int> m_renderTextures;
	std::vector<int> m_renderFloats;
	std::vector<SceneObject*> m_renderObjects;

	// Effect Pipeline Configurations - Allows pipeline customisation per shader
	UINT m_stencilRef;
	float m_blendFactor[4];
	ID3D11InputLayout* m_layout;	
	ID3D11BlendState* m_blendState;
	ID3D11DepthStencilState* m_depthStencilState;
	ID3D11RasterizerState* m_rasterizerState;
	ID3D11Buffer* m_matrixBuffer;
	ID3D11SamplerState* m_sampleState;

	// Effect Settings - These will usually be set in the shader implementation
	bool m_deferredShader = false;
	bool m_postProcessShader = false;
	bool m_boneAnimating = false;
	
	// use for moving the index count from the render implementation to the base render
	int m_indexCount;

	// use for moving the render type from the render implementation to the base render
	RenderType m_renderType = INDEXED;

	// thread count for attached compute shader
	int m_threadGroupCountX, m_threadGroupCountY, m_threadGroupCountZ;
};

#endif