#include "BilateralBlurShader.h"
#include "../DXFramework/ResourceContainer.h"

BilateralBlurShader::BilateralBlurShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	InitShader(L"../res/shaders/bilateral_blur_vs.hlsl", L"../res/shaders/bilateral_blur_ps.hlsl");
}

BilateralBlurShader::~BilateralBlurShader()
{
	if (m_psPerFrameBuffer)
	{
		m_psPerFrameBuffer->Release();
		m_psPerFrameBuffer = 0;
	}

}

void BilateralBlurShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	m_psPerFrameBuffer = D3D::instance()->CreateConstantBuffer(sizeof(PSPerFrameBufferType), true, true, NULL);

	// Initialise the two required sampler states
	D3D11_SAMPLER_DESC samplerDesc;
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_POINT;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_BORDER;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_BORDER;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_BORDER;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 1;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	m_device->CreateSamplerState(&samplerDesc, &m_normalDepthSampler);

	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;

	m_device->CreateSamplerState(&samplerDesc, &m_imageSampler);

	// create the depth enabled stencil state
	m_depthStencilState = D3D::instance()->CreateDefaultDepthStencilState();

	// create the alpha disabled blend state
	m_blendState = D3D::instance()->CreateDefaultBlendState();
	m_rasterizerState = D3D::instance()->CreateDefaultRasterizerState();
}

void BilateralBlurShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	PSPerFrameBufferType* perFrameBuffer;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	int blurCount = 8;
	for (int i = 0; i < blurCount; ++i)
	{
		// Set the blur output as the output merger target
		container->GetBuffers(m_renderBuffers[0])->SetRenderTargets();
		container->GetBuffers(m_renderBuffers[0])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 0.0f);

		if (m_renderBuffers.size() > 1)
		{
			ID3D11ShaderResourceView* blurTarget[] =
			{
				container->GetBuffers(m_renderBuffers[1])->GetShaderResourceView(0),
				container->GetBuffers(m_renderBuffers[2])->GetShaderResourceView(0)
			};
			deviceContext->PSSetShaderResources(1, 2, blurTarget);
		}
		else if (m_renderTextures.size() > 0)
		{
			// blur static images - unused
		}

		// Send the per frame data to the pixel shader
		deviceContext->Map(m_psPerFrameBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
		perFrameBuffer = (PSPerFrameBufferType*)mappedResource.pData;
		perFrameBuffer->texelWidth = 1.0f / (float)d3d->GetScreenWidth();
		perFrameBuffer->texelHeight = 1.0f / (float)d3d->GetScreenHeight();
		perFrameBuffer->horizontalBlur = 0.0f;
		perFrameBuffer->padding = 0.0f;
		deviceContext->Unmap(m_psPerFrameBuffer, 0);
		deviceContext->PSSetConstantBuffers(0, 1, &m_psPerFrameBuffer);

		// Set the pixel shader buffer 2 to null
		ID3D11Buffer* nullbuffer[] = { NULL };
		deviceContext->PSSetConstantBuffers(2, 1, nullbuffer);

		// Set the samplers in the pixel shader
		deviceContext->PSSetSamplers(0, 1, &m_normalDepthSampler);
		deviceContext->PSSetSamplers(1, 1, &m_imageSampler);

		for (auto object : m_renderObjects)
		{
			for (auto mesh : object->GetMeshes())
			{
				container->GetMesh(mesh)->SendData(deviceContext);
				m_indexCount = container->GetMesh(mesh)->GetIndexCount();
				m_renderType = container->GetMesh(mesh)->GetRenderType();

				BaseShader::Render(deviceContext, d3d, resourceContainer);
			}
		}

		ID3D11ShaderResourceView* nullSRVs[] = { NULL, NULL, NULL };
		deviceContext->PSSetShaderResources(0, 3, nullSRVs);

		// Set the original buffer as the output merger target
		container->GetBuffers(m_renderBuffers[1])->SetRenderTargets();
		container->GetBuffers(m_renderBuffers[1])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 0.0f);

		// Send the per frame data to the pixel shader
		deviceContext->Map(m_psPerFrameBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
		perFrameBuffer = (PSPerFrameBufferType*)mappedResource.pData;
		perFrameBuffer->texelWidth = 1.0f / (float)d3d->GetScreenWidth();
		perFrameBuffer->texelHeight = 1.0f / (float)d3d->GetScreenHeight();
		perFrameBuffer->horizontalBlur = 1.0f;
		perFrameBuffer->padding = 0.0f;
		deviceContext->Unmap(m_psPerFrameBuffer, 0);
		deviceContext->PSSetConstantBuffers(0, 1, &m_psPerFrameBuffer);

		ID3D11ShaderResourceView* blurTarget[] =
		{
			container->GetBuffers(m_renderBuffers[0])->GetShaderResourceView(0),
			container->GetBuffers(m_renderBuffers[2])->GetShaderResourceView(0)
		};
		deviceContext->PSSetShaderResources(1, 2, blurTarget);

		for (auto object : m_renderObjects)
		{
			for (auto mesh : object->GetMeshes())
			{
				container->GetMesh(mesh)->SendData(deviceContext);
				m_indexCount = container->GetMesh(mesh)->GetIndexCount();
				m_renderType = container->GetMesh(mesh)->GetRenderType();

				BaseShader::Render(deviceContext, d3d, resourceContainer);
			}
		}

		deviceContext->PSSetShaderResources(0, 3, nullSRVs);
	}

	ClearPipeline(deviceContext);
}