#include "camera.h"

Camera::Camera(float fov, float viewWidth, float viewHeight, float nearClip, float farClip)
{
	m_positionX = 0.0f;
	m_positionY = 0.0f;
	m_positionZ = 0.0f;

	m_rotationX = 0.0f;
	m_rotationY = 0.0f;
	m_rotationZ = 0.0f;

	m_fov = (fov * XM_PI) / 180.0f;
	m_viewWidth = viewWidth;
	m_viewHeight = viewHeight;
	m_nearClip = nearClip;
	m_farClip = farClip;
	m_aspect = viewWidth / viewHeight;

	GenerateProjectionMatrices();
}

Camera::Camera(const Camera& copy)
{

}

Camera::~Camera()
{
}

void Camera::SetFrameTime(float t)
{
	m_frameTime = t;
}

void Camera::SetPosition(float x, float y, float z)
{
	m_positionX = x;
	m_positionY = y;
	m_positionZ = z;
}

void Camera::SetRotation(float x, float y, float z)
{
	m_rotationX = x;
	m_rotationY = y;
	m_rotationZ = z;
}

XMFLOAT3 Camera::GetPosition()
{
	//XMVECTOR pos;
	//pos = XMVectorSet(m_positionX, m_positionY, m_positionZ, 1.0f);
	XMFLOAT3 pos(m_positionX, m_positionY, m_positionZ);
	return pos;
}

XMVECTOR Camera::GetRotation()
{
	XMVECTOR rot;
	rot = XMVectorSet(m_rotationX, m_rotationY, m_rotationZ, 1.0f);
	return rot;
}

void Camera::Update()
{
	XMVECTOR up, position, lookAt;
	float yaw, pitch, roll;
	XMMATRIX rotationMatrix;


	// Setup the vector that points upwards.
	up = XMVectorSet(0.0f, 1.0, 0.0, 1.0f);

	// Setup the position of the camera in the world.
	position = XMVectorSet(m_positionX, m_positionY, m_positionZ, 1.0f);
	
	// Setup where the camera is looking by default.
	lookAt = XMVectorSet(0.0, 0.0, 1.0f, 1.0f);
	
	// Set the yaw (Y axis), pitch (X axis), and roll (Z axis) rotations in radians.
	pitch = m_rotationX * 0.0174532925f;
	yaw = m_rotationY * 0.0174532925f;
	roll = m_rotationZ * 0.0174532925f;

	// Create the rotation matrix from the yaw, pitch, and roll values.
	rotationMatrix = XMMatrixRotationRollPitchYaw(pitch, yaw, roll);

	// Transform the lookAt and up vector by the rotation matrix so the view is correctly rotated at the origin.
	lookAt = XMVector3TransformCoord(lookAt, rotationMatrix);
	up = XMVector3TransformCoord(up, rotationMatrix);
	

	// Translate the rotated camera position to the location of the viewer.
	lookAt = position + lookAt;

	// Finally create the view matrix from the three updated vectors.
	m_viewMatrix = XMMatrixLookAtLH(position, lookAt, up);
}


void Camera::GetViewMatrix(XMMATRIX& viewMatrix)
{
	viewMatrix = m_viewMatrix;
}

void Camera::GetBaseViewMatrix(XMMATRIX& baseMatrix)
{
	XMVECTOR up, position, lookAt;

	// Setup the vector that points upwards.
	up = XMVectorSet(0.0f, 1.0, 0.0, 1.0f);
	
	// Setup the position of the camera in the world.
	position = XMVectorSet(0.0f, 0.0, -10.0, 1.0f);
	
	// Setup where the camera is looking by default.
	lookAt = XMVectorSet(0.0f, 0.0f, 0.0f, 1.0);
	
	// Finally create the view matrix from the three updated vectors.
	baseMatrix = XMMatrixLookAtLH(position, lookAt, up);
}

void Camera::GetProjectionMatrix(XMMATRIX& projectionMatrix)
{
	projectionMatrix = m_projectionMatrix;
}

void Camera::GetOrthographicMatrix(XMMATRIX& orthoMatrix)
{
	orthoMatrix = m_orthoMatrix;
}

void Camera::SetViewResolution(float viewWidth, float viewHeight)
{
	m_viewWidth = viewWidth;
	m_viewHeight = viewHeight;

	GenerateProjectionMatrices();
}

void Camera::SetFOV(float fov)
{
	m_fov = fov;

	GenerateProjectionMatrices();
}

void Camera::GenerateProjectionMatrices()
{
	// setup the camera projection matrix
	m_projectionMatrix = XMMatrixPerspectiveFovLH(m_fov, m_aspect, m_nearClip, m_farClip);

	// create the camera's orthographic projection matrix
	m_orthoMatrix = XMMatrixOrthographicLH(m_viewWidth, m_viewHeight, m_nearClip, m_farClip);
}

void Camera::MoveForward()
{
	float radians;

	// Update the forward movement based on the frame time
	m_speed = m_frameTime * 5.f;
	
	// Convert degrees to radians.
	radians = m_rotationY * 0.0174532925f;

	// Update the position.
	m_positionX += sinf(radians) * m_speed;
	m_positionZ += cosf(radians) * m_speed;
}


void Camera::MoveBackward()
{
	float radians;

	// Update the backward movement based on the frame time
	m_speed = m_frameTime * 5.f;// *0.5f;

	// Convert degrees to radians.
	radians = m_rotationY * 0.0174532925f;

	// Update the position.
	m_positionX -= sinf(radians) * m_speed;
	m_positionZ -= cosf(radians) * m_speed;
}


void Camera::MoveUpward()
{
	// Update the upward movement based on the frame time
	m_speed = m_frameTime * 5.f;// *0.5f;
	
	// Update the height position.
	m_positionY += m_speed;
}


void Camera::MoveDownward()
{
	// Update the downward movement based on the frame time
	m_speed = m_frameTime * 5.f;// *0.5f;

	// Update the height position.
	m_positionY -= m_speed;
}


void Camera::TurnLeft()
{
	// Update the left turn movement based on the frame time 
	m_speed = m_frameTime * 25.0f;
	
	// Update the rotation.
	m_rotationY -= m_speed;

	// Keep the rotation in the 0 to 360 range.
	if (m_rotationY < 0.0f)
	{
		m_rotationY += 360.0f;
	}
}


void Camera::TurnRight()
{
	// Update the right turn movement based on the frame time
	m_speed = m_frameTime * 25.0f;
	
	// Update the rotation.
	m_rotationY += m_speed;

	// Keep the rotation in the 0 to 360 range.
	if (m_rotationY > 360.0f)
	{
		m_rotationY -= 360.0f;
	}

}


void Camera::TurnUp()
{
	// Update the upward rotation movement based on the frame time
	m_speed = m_frameTime * 25.0f;
	
	// Update the rotation.
	m_rotationX -= m_speed;

	// Keep the rotation maximum 90 degrees.
	if (m_rotationX > 90.0f)
	{
		m_rotationX = 90.0f;
	}
}


void Camera::TurnDown()
{
	// Update the downward rotation movement based on the frame time
	m_speed = m_frameTime * 25.0f;

	// Update the rotation.
	m_rotationX += m_speed;

	// Keep the rotation maximum 90 degrees.
	if (m_rotationX < -90.0f)
	{
		m_rotationX = -90.0f;
	}
}


void Camera::Turn(int x, int y)
{
	// Update the right turn movement based on the frame time
	m_speed = m_frameTime * 25.0f;

	// Update the rotation.
	m_rotationY += m_speed * x;

	// Keep the rotation in the 0 to 360 range.
	if (m_rotationY > 360.0f)
	{
		m_rotationY -= 360.0f;
	}
	else if (m_rotationY < 0.f)
	{
		m_rotationY += 360.0f;
	}

	m_rotationX += m_speed * y;
	// Keep the rotation in the 0 to 360 range.
	if (m_rotationX > 360.0f)
	{
		m_rotationX -= 360.0f;
	}
	else if (m_rotationX < 0.f)
	{
		m_rotationX += 360.0f;
	}
}

void Camera::StrafeRight()
{
	float radians;

	// Update the forward movement based on the frame time
	m_speed = m_frameTime * 5.f;

	// Convert degrees to radians.
	radians = m_rotationY * 0.0174532925f;

	// Update the position.
	m_positionZ -= sinf(radians) * m_speed;
	m_positionX += cosf(radians) * m_speed;

}

void Camera::StrafeLeft()
{
	float radians;

	// Update the forward movement based on the frame time
	m_speed = m_frameTime * 5.f;

	// Convert degrees to radians.
	radians = m_rotationY * 0.0174532925f;

	// Update the position.
	m_positionZ += sinf(radians) * m_speed;
	m_positionX -= cosf(radians) * m_speed;
}

float Camera::GetFOV()
{
	return m_fov;
}

float Camera::GetNearClip()
{
	return m_nearClip;
}

float Camera::GetFarClip()
{
	return m_farClip;
}