#include "D3D.h"
#include "BufferScalingShader.h"

D3D* D3D::m_Direct3DHandle = 0;

void D3D::init(int windowWidth, int windowHeight, int resolutionWidth, int resolutionHeight, bool vsync, HWND hwnd, bool fullscreen, AntiAliasingSetting aaSetting)
{
	HRESULT result;
	IDXGIFactory* factory;
	IDXGIAdapter* adapter;
	IDXGIOutput* adapterOutput;
	unsigned int numModes, i, numerator, denominator, stringLength;
	DXGI_MODE_DESC* displayModeList;
	DXGI_ADAPTER_DESC adapterDesc;
	DXGI_SWAP_CHAIN_DESC swapChainDesc;
	D3D_FEATURE_LEVEL featureLevel;
	ID3D11Texture2D* backBufferPtr;
	ID3D11Texture2D* intermediateBufferPtr;
	D3D11_TEXTURE2D_DESC depthBufferDesc;
	D3D11_DEPTH_STENCIL_DESC depthStencilDesc;
	D3D11_DEPTH_STENCIL_VIEW_DESC depthStencilViewDesc;
	D3D11_TEXTURE2D_DESC textureDesc;
	D3D11_RENDER_TARGET_VIEW_DESC renderTargetViewDesc;
	D3D11_SHADER_RESOURCE_VIEW_DESC shaderResourceViewDesc;

	m_aaSetting = aaSetting;

	// set the sample count for msaa based on the aa setting
	switch (aaSetting)
	{
	case AntiAliasingSetting::MSAA2X:
	{
		m_sampleCount = 2;
	}
	break;
	case AntiAliasingSetting::MSAA4X:
	{
		m_sampleCount = 4;
	}
	break;
	case AntiAliasingSetting::MSAA8X:
	{
		m_sampleCount = 8;
	}
	break;
	case AntiAliasingSetting::MSAA16X:
	{
		m_sampleCount = 16;
	}
	break;
	default:
	{
		m_sampleCount = 1;
	}
	break;
	}
	
	// store the resolution data
	m_windowWidth = windowWidth;
	m_windowHeight = windowHeight;

	m_resolutionWidth = resolutionWidth;
	m_resolutionHeight = resolutionHeight;

	// store the vsync setting
	m_vsync_enabled = vsync;

	// Create a DirectX graphics interface factory.
	CreateDXGIFactory(__uuidof(IDXGIFactory), (void**)&factory);

	// Use the factory to create an adapter for the primary graphics interface (video card).
	factory->EnumAdapters(0, &adapter);

	// Enumerate the primary adapter output (monitor).
	adapter->EnumOutputs(0, &adapterOutput);

	// Get the number of modes that fit the DXGI_FORMAT_R8G8B8A8_UNORM display format for the adapter output (monitor).
	adapterOutput->GetDisplayModeList(DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_ENUM_MODES_INTERLACED, &numModes, NULL);

	// Create a list to hold all the possible display modes for this monitor/video card combination.
	displayModeList = new DXGI_MODE_DESC[numModes];

	// Now fill the display mode list structures.
	adapterOutput->GetDisplayModeList(DXGI_FORMAT_R8G8B8A8_UNORM, DXGI_ENUM_MODES_INTERLACED, &numModes, displayModeList);
	
	// Now go through all the display modes and find the one that matches the screen width and height.
	// When a match is found store the numerator and denominator of the refresh rate for that monitor.
	for (i = 0; i<numModes; i++)
	{
		if (displayModeList[i].Width == (unsigned int)windowWidth)
		{
			if (displayModeList[i].Height == (unsigned int)windowHeight)
			{
				numerator = displayModeList[i].RefreshRate.Numerator;
				denominator = displayModeList[i].RefreshRate.Denominator;
			}
		}
	}

	// Get the adapter (video card) description.
	adapter->GetDesc(&adapterDesc);

	// Store the dedicated video card memory in megabytes.
	m_videoCardMemory = (int)(adapterDesc.DedicatedVideoMemory / 1024 / 1024);

	// Convert the name of the video card to a character array and store it.
	//wcstombs_s(&stringLength, m_videoCardDescription, 128, adapterDesc.Description, 128);

	// Release the display mode list.
	delete[] displayModeList;
	displayModeList = 0;

	// Release the adapter output.
	adapterOutput->Release();
	adapterOutput = 0;

	// Release the adapter.
	adapter->Release();
	adapter = 0;

	// Release the factory.
	factory->Release();
	factory = 0;

	// Initialize the swap chain description.
	ZeroMemory(&swapChainDesc, sizeof(swapChainDesc));

	// Set to a single back buffer.
	swapChainDesc.BufferCount = 1;

	// Set the width and height of the back buffer.
	swapChainDesc.BufferDesc.Width = windowWidth;
	swapChainDesc.BufferDesc.Height = windowHeight;

	// Set regular 32-bit surface for the back buffer.
	swapChainDesc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;

	// Set the refresh rate of the back buffer.
	if (m_vsync_enabled)
	{
		swapChainDesc.BufferDesc.RefreshRate.Numerator = numerator;
		swapChainDesc.BufferDesc.RefreshRate.Denominator = denominator;
	}
	else
	{
		swapChainDesc.BufferDesc.RefreshRate.Numerator = 0;
		swapChainDesc.BufferDesc.RefreshRate.Denominator = 1;
	}

	// Set the usage of the back buffer.
	swapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;

	// Set the handle for the window to render to.
	swapChainDesc.OutputWindow = hwnd;

	// Turn multisampling on.
	swapChainDesc.SampleDesc.Count = m_sampleCount;
	swapChainDesc.SampleDesc.Quality = 0;

	// Set to full screen or windowed mode
	if (fullscreen)
	{
		swapChainDesc.Windowed = false;
	}
	else
	{
		swapChainDesc.Windowed = true;
	}

	// Set the scan line ordering and scaling to unspecified.
	swapChainDesc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
	swapChainDesc.BufferDesc.Scaling = DXGI_MODE_SCALING_UNSPECIFIED;

	// Discard the back buffer contents after presenting.
	swapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;

	// Don't set the advanced flags.
	swapChainDesc.Flags = 0;

	// Set the feature level to DirectX 11.
	featureLevel = D3D_FEATURE_LEVEL_11_0;

	// Create the swap chain, Direct3D device, and Direct3D device context.
	result = D3D11CreateDeviceAndSwapChain(NULL, D3D_DRIVER_TYPE_HARDWARE, NULL, D3D11_CREATE_DEVICE_DEBUG, &featureLevel, 1, D3D11_SDK_VERSION, &swapChainDesc, &m_swapChain, &m_device, NULL, &m_deviceContext);
	if (FAILED(result))
	{
		 D3D11CreateDeviceAndSwapChain(NULL, D3D_DRIVER_TYPE_HARDWARE, NULL, 0, &featureLevel, 1, D3D11_SDK_VERSION, &swapChainDesc, &m_swapChain, &m_device, NULL, &m_deviceContext);
	}

	// Get the pointer to the back buffer.
	m_swapChain->GetBuffer(0, __uuidof(ID3D11Texture2D), (LPVOID*)&backBufferPtr);

	// Create the render target view for the back buffer
	m_device->CreateRenderTargetView(backBufferPtr, NULL, &m_backBufferTargetView);

	// Release pointer to the back buffer
	backBufferPtr->Release();
	backBufferPtr = 0;

	// Initialize the description of the depth buffer.
	ZeroMemory(&depthBufferDesc, sizeof(depthBufferDesc));

	// Set up the description of the depth buffer.
	depthBufferDesc.Width = windowWidth;
	depthBufferDesc.Height = windowHeight;
	depthBufferDesc.MipLevels = 1;
	depthBufferDesc.ArraySize = 1;
	depthBufferDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthBufferDesc.SampleDesc.Count = m_sampleCount;
	depthBufferDesc.SampleDesc.Quality = 0;
	depthBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	depthBufferDesc.BindFlags = D3D11_BIND_DEPTH_STENCIL;
	depthBufferDesc.CPUAccessFlags = 0;
	depthBufferDesc.MiscFlags = 0;

	// Create the texture for the depth buffer using the filled out description.
	m_device->CreateTexture2D(&depthBufferDesc, NULL, &m_backBufferDepthStencilBuffer);

	// Initialize the depth stencil view.
	ZeroMemory(&depthStencilViewDesc, sizeof(depthStencilViewDesc));

	// Set up the depth stencil view description.
	depthStencilViewDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthStencilViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_DSV_DIMENSION_TEXTURE2DMS : D3D11_DSV_DIMENSION_TEXTURE2D;
	if (m_sampleCount == 1)depthStencilViewDesc.Texture2D.MipSlice = 0;

	// Create the depth stencil view.
	result = m_device->CreateDepthStencilView(m_backBufferDepthStencilBuffer, &depthStencilViewDesc, &m_backBufferDepthStencilView);

	// create the back buffer viewport
	m_backBufferViewport.Width = (float)windowWidth;
	m_backBufferViewport.Height = (float)windowHeight;
	m_backBufferViewport.MinDepth = 0.0f;
	m_backBufferViewport.MaxDepth = 1.0f;
	m_backBufferViewport.TopLeftX = 0.0f;
	m_backBufferViewport.TopLeftY = 0.0f;

	// set up the description for the intermediate buffer
	ZeroMemory(&textureDesc, sizeof(textureDesc));
	textureDesc.Width = resolutionWidth;
	textureDesc.Height = resolutionHeight;
	textureDesc.MipLevels = 1;
	textureDesc.ArraySize = 1;
	textureDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	textureDesc.SampleDesc.Count = m_sampleCount;
	textureDesc.SampleDesc.Quality = 0;
	textureDesc.Usage = D3D11_USAGE_DEFAULT;
	textureDesc.BindFlags = D3D11_BIND_RENDER_TARGET | D3D11_BIND_SHADER_RESOURCE;
	textureDesc.CPUAccessFlags = 0;
	textureDesc.MiscFlags = 0;

	// create the intermediate buffer
	m_device->CreateTexture2D(&textureDesc, NULL, &intermediateBufferPtr);

	// setup the render target view for the intermediate buffer
	renderTargetViewDesc.Format = textureDesc.Format;
	renderTargetViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_RTV_DIMENSION_TEXTURE2DMS : D3D11_RTV_DIMENSION_TEXTURE2D;
	renderTargetViewDesc.Texture2D.MipSlice = 0;

	// create the intermediate render target view
	m_device->CreateRenderTargetView(intermediateBufferPtr, &renderTargetViewDesc, &m_intermediateBufferTargetView);

	// setup the shade resource view for the intermediate buffer
	shaderResourceViewDesc.Format = textureDesc.Format;
	shaderResourceViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_SRV_DIMENSION_TEXTURE2DMS : D3D11_SRV_DIMENSION_TEXTURE2D;
	shaderResourceViewDesc.Texture2D.MostDetailedMip = 0;
	shaderResourceViewDesc.Texture2D.MipLevels = 1;

	// create the intermediate shader resource view
	m_device->CreateShaderResourceView(intermediateBufferPtr, &shaderResourceViewDesc, &m_intermediateBufferShaderResourceView);

	// initialize the description of the depth buffer
	ZeroMemory(&depthBufferDesc, sizeof(depthBufferDesc));
	depthBufferDesc.Width = resolutionWidth;
	depthBufferDesc.Height = resolutionHeight;
	depthBufferDesc.MipLevels = 1;
	depthBufferDesc.ArraySize = 1;
	depthBufferDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthBufferDesc.SampleDesc.Count = m_sampleCount;
	depthBufferDesc.SampleDesc.Quality = 0;
	depthBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	depthBufferDesc.BindFlags = D3D11_BIND_DEPTH_STENCIL;
	depthBufferDesc.CPUAccessFlags = 0;
	depthBufferDesc.MiscFlags = 0;

	// create the intermediate depth stencil buffer
	result = m_device->CreateTexture2D(&depthBufferDesc, NULL, &m_intermediateDepthStencilBuffer);
	
	// initialize the description of the depth stencil view
	ZeroMemory(&depthStencilViewDesc, sizeof(depthStencilViewDesc));
	depthStencilViewDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthStencilViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_DSV_DIMENSION_TEXTURE2DMS : D3D11_DSV_DIMENSION_TEXTURE2D;
	if (m_sampleCount == 1)depthStencilViewDesc.Texture2D.MipSlice = 0;

	result = m_device->CreateDepthStencilView(m_intermediateDepthStencilBuffer, &depthStencilViewDesc, &m_intermediateBufferDepthStencilView);

	// create the back buffer viewport
	m_intermediateBufferViewport.Width = (float)resolutionWidth;
	m_intermediateBufferViewport.Height = (float)resolutionHeight;
	m_intermediateBufferViewport.MinDepth = 0.0f;
	m_intermediateBufferViewport.MaxDepth = 1.0f;
	m_intermediateBufferViewport.TopLeftX = 0.0f;
	m_intermediateBufferViewport.TopLeftY = 0.0f;

	// create the scaling shader that is used to transfer the intermediate buffer to the back buffer
	m_bufferScalingShader = new BufferScalingShader(m_device, hwnd);
	reinterpret_cast<BufferScalingShader*>(m_bufferScalingShader)->SetBuffers(m_intermediateBufferShaderResourceView, m_backBufferTargetView);
}

D3D::~D3D()
{
	// before shutting down set to windowed mode before releasing the swap chain
	if (m_swapChain)
	{
		m_swapChain->SetFullscreenState(false, NULL);
	}

	// release the back buffer components
	if (m_backBufferDepthStencilView)
	{
		m_backBufferDepthStencilView->Release();
		m_backBufferDepthStencilView = 0;
	}

	if (m_backBufferDepthStencilBuffer)
	{
		m_backBufferDepthStencilBuffer->Release();
		m_backBufferDepthStencilBuffer = 0;
	}

	if (m_backBufferTargetView)
	{
		m_backBufferTargetView->Release();
		m_backBufferTargetView = 0;
	}

	// release the intermediate buffer components
	if (m_intermediateBufferDepthStencilView)
	{
		m_intermediateBufferDepthStencilView->Release();
		m_intermediateBufferDepthStencilView = 0;
	}

	if (m_intermediateDepthStencilBuffer)
	{
		m_intermediateDepthStencilBuffer->Release();
		m_intermediateDepthStencilBuffer = 0;
	}

	if (m_intermediateBufferTargetView)
	{
		m_intermediateBufferTargetView->Release();
		m_intermediateBufferTargetView = 0;
	}

	// release the system components
	if (m_deviceContext)
	{
		m_deviceContext->Release();
		m_deviceContext = 0;
	}

	if (m_device)
	{
		m_device->Release();
		m_device = 0;
	}

	if (m_swapChain)
	{
		m_swapChain->Release();
		m_swapChain = 0;
	}
}

void D3D::BeginScene(float red, float green, float blue, float alpha)
{
	float color[4];

	// setup the buffer clear color
	color[0] = red;
	color[1] = green;
	color[2] = blue;
	color[3] = alpha;

	// clear the intermediate and back buffers
	m_deviceContext->ClearRenderTargetView(m_backBufferTargetView, color);
	m_deviceContext->ClearRenderTargetView(m_intermediateBufferTargetView, color);

	// clear the depth buffers
	m_deviceContext->ClearDepthStencilView(m_backBufferDepthStencilView, D3D11_CLEAR_DEPTH, 1.0f, 0);
	m_deviceContext->ClearDepthStencilView(m_intermediateBufferDepthStencilView, D3D11_CLEAR_DEPTH, 1.0f, 0);
}

void D3D::EndScene()
{
	// scale the intermediate buffer into the back buffer before presenting
	reinterpret_cast<BufferScalingShader*>(m_bufferScalingShader)->Render(m_deviceContext, this, NULL);

	// present the back buffer to the screen
	if (m_vsync_enabled)
	{
		// lock to screen refresh rate
		m_swapChain->Present(1, 0);
	}
	else
	{
		// present as fast as possible
		m_swapChain->Present(0, 0);
	}
}

ID3D11Device* D3D::GetDevice()
{
	return m_device;
}

ID3D11DeviceContext* D3D::GetDeviceContext()
{
	return m_deviceContext;
}

void D3D::GetVideoCardInfo(char* cardName, int& memory)
{
	strcpy_s(cardName, 128, m_videoCardDescription);
	memory = m_videoCardMemory;
}

void D3D::SetIntermediateBufferRenderTarget()
{
	// bind the intermediate buffer render target view and depth stencil buffer
	m_deviceContext->OMSetRenderTargets(1, &m_intermediateBufferTargetView, m_intermediateBufferDepthStencilView);
}

void D3D::ResetViewport()
{
	// set the viewport
	m_deviceContext->RSSetViewports(1, &m_intermediateBufferViewport);
}

void D3D::SetBackBufferViewport()
{
	// set the viewport
	m_deviceContext->RSSetViewports(1, &m_backBufferViewport);
}

ID3D11Buffer* D3D::CreateVertexBuffer(UINT size, bool dynamic, bool streamout, D3D11_SUBRESOURCE_DATA* pData)
{
	D3D11_BUFFER_DESC desc;
	desc.ByteWidth = size;
	desc.MiscFlags = 0;
	desc.StructureByteStride = 0;

	if (streamout)
	{
		desc.BindFlags = D3D11_BIND_VERTEX_BUFFER | D3D11_BIND_STREAM_OUTPUT;
	}
	else
	{
		desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	}

	if (dynamic)
	{
		desc.Usage = D3D11_USAGE_DYNAMIC;
		desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
	}
	else
	{
		desc.Usage = D3D11_USAGE_IMMUTABLE;
		desc.CPUAccessFlags = 0;
	}

	ID3D11Buffer* pBuffer = 0;
	HRESULT result = m_device->CreateBuffer(&desc, pData, &pBuffer);
	if (FAILED(result))
	{
		return nullptr;
	}

	return pBuffer;
}

ID3D11Buffer* D3D::CreateIndexBuffer(UINT size, bool dynamic, D3D11_SUBRESOURCE_DATA* pData)
{
	D3D11_BUFFER_DESC desc;
	desc.ByteWidth = size;
	desc.MiscFlags = 0;
	desc.StructureByteStride = 0;
	desc.BindFlags = D3D11_BIND_INDEX_BUFFER;

	if (dynamic)
	{
		desc.Usage = D3D11_USAGE_DYNAMIC;
		desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
	}
	else
	{
		desc.Usage = D3D11_USAGE_IMMUTABLE;
		desc.CPUAccessFlags = 0;
	}

	ID3D11Buffer* pBuffer = 0;
	HRESULT result = m_device->CreateBuffer(&desc, pData, &pBuffer);
	if (FAILED(result))
	{
		return nullptr;
	}

	return pBuffer;
}

ID3D11Buffer* D3D::CreateConstantBuffer(UINT size, bool dynamic, bool CPUupdates, D3D11_SUBRESOURCE_DATA* pData)
{
	D3D11_BUFFER_DESC desc;
	desc.ByteWidth = size;
	desc.MiscFlags = 0;
	desc.StructureByteStride = 0;
	desc.BindFlags = D3D11_BIND_CONSTANT_BUFFER;

	if (dynamic && CPUupdates)
	{
		desc.Usage = D3D11_USAGE_DYNAMIC;
		desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
	}
	else if (dynamic && !CPUupdates)
	{
		desc.Usage = D3D11_USAGE_DEFAULT;
		desc.CPUAccessFlags = 0;
	}
	else
	{
		desc.Usage = D3D11_USAGE_IMMUTABLE;
		desc.CPUAccessFlags = 0;
	}

	ID3D11Buffer* pBuffer = 0;
	HRESULT hr = m_device->CreateBuffer(&desc, pData, &pBuffer);

	if (FAILED(hr))
	{
		return nullptr;
	}

	return pBuffer;
}

ID3D11DepthStencilState* D3D::CreateDefaultDepthStencilState()
{
	D3D11_DEPTH_STENCIL_DESC depthStencilDesc;
	ID3D11DepthStencilState* depthStencilState;

	// Initialize the description of the stencil state.
	ZeroMemory(&depthStencilDesc, sizeof(depthStencilDesc));

	// Set up the description of the stencil state.
	depthStencilDesc.DepthEnable = true;
	depthStencilDesc.DepthWriteMask = D3D11_DEPTH_WRITE_MASK_ALL;
	depthStencilDesc.DepthFunc = D3D11_COMPARISON_LESS;

	depthStencilDesc.StencilEnable = true;
	depthStencilDesc.StencilReadMask = 0xFF;
	depthStencilDesc.StencilWriteMask = 0xFF;

	// Stencil operations if pixel is front-facing.
	depthStencilDesc.FrontFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.FrontFace.StencilDepthFailOp = D3D11_STENCIL_OP_INCR;
	depthStencilDesc.FrontFace.StencilPassOp = D3D11_STENCIL_OP_REPLACE;
	depthStencilDesc.FrontFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	// Stencil operations if pixel is back-facing.
	depthStencilDesc.BackFace.StencilFailOp = D3D11_STENCIL_OP_KEEP;
	depthStencilDesc.BackFace.StencilDepthFailOp = D3D11_STENCIL_OP_DECR;
	depthStencilDesc.BackFace.StencilPassOp = D3D11_STENCIL_OP_REPLACE;
	depthStencilDesc.BackFace.StencilFunc = D3D11_COMPARISON_ALWAYS;

	// Create the depth stencil state.
	m_device->CreateDepthStencilState(&depthStencilDesc, &depthStencilState);

	return depthStencilState;
}

ID3D11BlendState* D3D::CreateDefaultBlendState()
{
	D3D11_BLEND_DESC blendStateDescription;
	ID3D11BlendState* blendState;

	// Clear the blend state description.
	ZeroMemory(&blendStateDescription, sizeof(D3D11_BLEND_DESC));

	// Create an alpha disabled blend state description.
	blendStateDescription.RenderTarget[0].BlendEnable = FALSE;
	blendStateDescription.RenderTarget[0].SrcBlend = D3D11_BLEND_ONE;
	blendStateDescription.RenderTarget[0].DestBlend = D3D11_BLEND_INV_SRC_COLOR;
	blendStateDescription.RenderTarget[0].BlendOp = D3D11_BLEND_OP_ADD;
	blendStateDescription.RenderTarget[0].SrcBlendAlpha = D3D11_BLEND_ONE;
	blendStateDescription.RenderTarget[0].DestBlendAlpha = D3D11_BLEND_ZERO;
	blendStateDescription.RenderTarget[0].BlendOpAlpha = D3D11_BLEND_OP_ADD;
	blendStateDescription.RenderTarget[0].RenderTargetWriteMask = 0x0f;

	// Create the blend state using the description.
	m_device->CreateBlendState(&blendStateDescription, &blendState);

	return blendState;
}


ID3D11RasterizerState* D3D::CreateDefaultRasterizerState()
{
	D3D11_RASTERIZER_DESC rasterDesc;
	ID3D11RasterizerState* rasterState;
	
	// Clear the blend state description.
	ZeroMemory(&rasterDesc, sizeof(D3D11_RASTERIZER_DESC));

	// Setup the raster description which will determine how and what polygons will be drawn.
	rasterDesc.AntialiasedLineEnable = true;
	rasterDesc.CullMode = D3D11_CULL_BACK;
	rasterDesc.DepthBias = 0;
	rasterDesc.DepthBiasClamp = 0.0f;
	rasterDesc.DepthClipEnable = true;
	rasterDesc.FillMode = D3D11_FILL_SOLID;
	rasterDesc.FrontCounterClockwise = true;
	rasterDesc.MultisampleEnable = true;
	rasterDesc.ScissorEnable = false;
	rasterDesc.SlopeScaledDepthBias = 0.0f;

	// Create the rasterizer state from the description we just filled out.
	m_device->CreateRasterizerState(&rasterDesc, &rasterState);

	return rasterState;
}

ID3D11DepthStencilView* D3D::GetDepthStencilView()
{
	return m_intermediateBufferDepthStencilView;
}

ID3D11ShaderResourceView* D3D::GetIntermediateBufferResourceView()
{
	return m_intermediateBufferShaderResourceView;
}

ID3D11RenderTargetView* D3D::GetIntermediateBufferRenderTarget()
{
	return m_intermediateBufferTargetView;
}

int D3D::GetScreenWidth()
{
	return m_windowWidth;
}

int D3D::GetScreenHeight()
{
	return m_windowHeight;
}

int D3D::GetResolutionWidth()
{
	return m_resolutionWidth;
}

int D3D::GetResolutionHeight()
{
	return m_resolutionHeight;
}

AntiAliasingSetting D3D::GetAntiAliasingSetting()
{
	return m_aaSetting;
}
