#include "DeferredBuffers.h"

DeferredBuffers::DeferredBuffers(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int screenWidth, int screenHeight, int bufferCount, int subsamples)
{
	m_sampleCount = subsamples;
	m_device = device;
	m_deviceContext = deviceContext;
	m_bufferWidth = screenWidth;
	m_bufferHeight = screenHeight;
	m_gBufferCount = bufferCount;
	InitGBuffer(screenWidth, screenHeight);
}

DeferredBuffers::~DeferredBuffers()
{
	//release the G-Buffer components
	for (int i = 0; i < m_gBufferCount; i++)
	{
		if (m_gBufferRenderTexture[i])
		{
			m_gBufferRenderTexture[i]->Release();
			m_gBufferRenderTexture[i] = nullptr;
		}

		if (m_gBufferResourceView[i])
		{
			m_gBufferResourceView[i]->Release();
			m_gBufferResourceView[i] = nullptr;
		}

		if (m_gBufferTargetView[i])
		{
			m_gBufferTargetView[i]->Release();
			m_gBufferTargetView[i] = nullptr;
		}
	}
}

void DeferredBuffers::InitGBuffer(int screenWidth, int screenHeight)
{
	D3D11_TEXTURE2D_DESC textureDesc;
	D3D11_RENDER_TARGET_VIEW_DESC renderTargetViewDesc;
	D3D11_SHADER_RESOURCE_VIEW_DESC shaderResourceViewDesc;
	D3D11_TEXTURE2D_DESC depthBufferDesc;
	D3D11_DEPTH_STENCIL_VIEW_DESC depthStencilViewDesc;
	HRESULT result;

	ZeroMemory(&textureDesc, sizeof(textureDesc));
	textureDesc.Width = screenWidth;
	textureDesc.Height = screenHeight;
	textureDesc.MipLevels = 1;
	textureDesc.ArraySize = 1;
	textureDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	textureDesc.SampleDesc.Count = m_sampleCount;
	textureDesc.SampleDesc.Quality = 0;
	textureDesc.Usage = D3D11_USAGE_DEFAULT;
	textureDesc.BindFlags = D3D11_BIND_RENDER_TARGET | D3D11_BIND_SHADER_RESOURCE;
	textureDesc.CPUAccessFlags = 0;
	textureDesc.MiscFlags = 0;

	m_gBufferRenderTexture = new ID3D11Texture2D*[m_gBufferCount];

	for (int i = 0; i < m_gBufferCount; i++)
	{
		m_device->CreateTexture2D(&textureDesc, NULL, &m_gBufferRenderTexture[i]);
	}

	renderTargetViewDesc.Format = textureDesc.Format;
	renderTargetViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_RTV_DIMENSION_TEXTURE2DMS : D3D11_RTV_DIMENSION_TEXTURE2D;
	renderTargetViewDesc.Texture2D.MipSlice = 0;

	m_gBufferTargetView = new ID3D11RenderTargetView*[m_gBufferCount];

	for (int i = 0; i < m_gBufferCount; i++)
	{
		m_device->CreateRenderTargetView(m_gBufferRenderTexture[i], &renderTargetViewDesc, &m_gBufferTargetView[i]);
	}

	shaderResourceViewDesc.Format = textureDesc.Format;
	shaderResourceViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_SRV_DIMENSION_TEXTURE2DMS : D3D11_SRV_DIMENSION_TEXTURE2D;
	shaderResourceViewDesc.Texture2D.MostDetailedMip = 0;
	shaderResourceViewDesc.Texture2D.MipLevels = 1;

	m_gBufferResourceView = new ID3D11ShaderResourceView*[m_gBufferCount];

	for (int i = 0; i < m_gBufferCount; i++)
	{
		m_device->CreateShaderResourceView(m_gBufferRenderTexture[i], &shaderResourceViewDesc, &m_gBufferResourceView[i]);
	}

	UINT sampleQuality;
	m_device->CheckMultisampleQualityLevels(DXGI_FORMAT_D24_UNORM_S8_UINT, m_sampleCount, &sampleQuality);

	// Initialize the description of the depth bufffer
	ZeroMemory(&depthBufferDesc, sizeof(depthBufferDesc));
	depthBufferDesc.Width = screenWidth;
	depthBufferDesc.Height = screenHeight;
	depthBufferDesc.MipLevels = 1;
	depthBufferDesc.ArraySize = 1;
	depthBufferDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthBufferDesc.SampleDesc.Count = m_sampleCount;
	depthBufferDesc.SampleDesc.Quality = 0;
	depthBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	depthBufferDesc.BindFlags = D3D11_BIND_DEPTH_STENCIL;
	depthBufferDesc.CPUAccessFlags = 0;
	depthBufferDesc.MiscFlags = 0;

	result = m_device->CreateTexture2D(&depthBufferDesc, NULL, &m_depthStencilBuffer);

	ZeroMemory(&depthStencilViewDesc, sizeof(depthStencilViewDesc));
	depthStencilViewDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
	depthStencilViewDesc.ViewDimension = (m_sampleCount > 1) ? D3D11_DSV_DIMENSION_TEXTURE2DMS : D3D11_DSV_DIMENSION_TEXTURE2D;
	if(m_sampleCount == 1)depthStencilViewDesc.Texture2D.MipSlice = 0;

	result = m_device->CreateDepthStencilView(m_depthStencilBuffer, &depthStencilViewDesc, &m_depthStencilView);

	m_viewport.Width = (float)screenWidth;
	m_viewport.Height = (float)screenHeight;
	m_viewport.MinDepth = 0.0f;
	m_viewport.MaxDepth = 1.0f;
	m_viewport.TopLeftX = 0.0f;
	m_viewport.TopLeftY = 0.0f;
}

void DeferredBuffers::SetRenderTargets(ID3D11DepthStencilView* stencilView)
{
	m_deviceContext->OMSetRenderTargets(m_gBufferCount, m_gBufferTargetView, (stencilView) ? stencilView : m_depthStencilView);
	m_deviceContext->RSSetViewports(1, &m_viewport);
}

void DeferredBuffers::ClearRenderTargets(float r, float g, float b, float a)
{
	float color[4];

	color[0] = r;
	color[1] = g;
	color[2] = b;
	color[3] = a;

	for (int i = 0; i < m_gBufferCount; i++)
	{
		m_deviceContext->ClearRenderTargetView(m_gBufferTargetView[i], color);
	}

	m_deviceContext->ClearDepthStencilView(m_depthStencilView, D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL, 1.0f, 0);
}

ID3D11ShaderResourceView* DeferredBuffers::GetShaderResourceView(int viewNum)
{
	if(viewNum < m_gBufferCount)
		return m_gBufferResourceView[viewNum];
}

ID3D11DepthStencilView* DeferredBuffers::GetDepthStencilView()
{
	return m_depthStencilView;
}

void DeferredBuffers::GetBufferDimensions(int& width, int& height)
{
	width = m_bufferWidth;
	height = m_bufferHeight;
}

int DeferredBuffers::GetBufferCount()
{
	return m_gBufferCount;
}