#include "DepthShader.h"
#include "../DXFramework/ResourceContainer.h"

DepthShader::DepthShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	InitShader(L"../res/shaders/depth_vs.hlsl", L"../res/shaders/depth_ps.hlsl");
}

DepthShader::~DepthShader()
{
	if (m_matrixBuffer)
	{
		m_matrixBuffer->Release();
		m_matrixBuffer = 0;
	}

	BaseShader::~BaseShader();
}

void DepthShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	m_matrixBuffer = D3D::instance()->CreateConstantBuffer(sizeof(VSMatrixBufferType), true, true, NULL);
	m_psMatrixBuffer = D3D::instance()->CreateConstantBuffer(sizeof(PSMatrixBufferType), true, true, NULL);
	
	// Create a texture sampler state description.
	D3D11_SAMPLER_DESC samplerDesc;
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 1;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// Create the texture sampler state.
	m_device->CreateSamplerState(&samplerDesc, &m_sampleState);

	// create the depth enabled stencil state
	m_depthStencilState = D3D::instance()->CreateDefaultDepthStencilState();

	// create the alpha disabled blend state
	m_blendState = D3D::instance()->CreateDefaultBlendState();
	m_rasterizerState = D3D::instance()->CreateDefaultRasterizerState();

}

void DepthShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	HRESULT result;
	VSMatrixBufferType* matrixBuffer;
	PSMatrixBufferType* psMatrixBuffer;
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	XMMATRIX world, view, proj;
	int bufferNumber;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	// Set the depth normal buffer as the output for the pixel shader
	container->GetBuffers(m_renderBuffers[0])->SetRenderTargets();
	container->GetBuffers(m_renderBuffers[0])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 0.0f);

	for (auto camera : m_renderCameras)
	{
		container->GetCamera(camera)->GetViewMatrix(view);
		container->GetCamera(camera)->GetProjectionMatrix(proj);

		for (auto object : m_renderObjects)
		{
			world = object->GetTransformMatrix();

			// Send the matrix data to the vertex shader
			result = deviceContext->Map(m_matrixBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
			matrixBuffer = (VSMatrixBufferType*)mappedResource.pData;
			matrixBuffer->world = XMMatrixTranspose(world);
			matrixBuffer->view = XMMatrixTranspose(view);
			matrixBuffer->proj = XMMatrixTranspose(proj);
			deviceContext->Unmap(m_matrixBuffer, 0);
			bufferNumber = 0;
			deviceContext->VSSetConstantBuffers(bufferNumber, 1, &m_matrixBuffer);

			// Send the matrix data to the pixel shader
			result = deviceContext->Map(m_psMatrixBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);
			psMatrixBuffer = (PSMatrixBufferType*)mappedResource.pData;
			psMatrixBuffer->invView = XMMatrixTranspose(XMMatrixInverse(NULL, view));
			deviceContext->Unmap(m_psMatrixBuffer, 0);
			bufferNumber = 0;
			deviceContext->PSSetConstantBuffers(bufferNumber, 1, &m_psMatrixBuffer);

			// Set shader texture resource in the pixel shader.
			ID3D11ShaderResourceView* texture = container->GetTexture(object->GetTextureIndex(1))->GetTexture();
			deviceContext->PSSetShaderResources(0, 1, &texture);

			// Set the sampler state
			deviceContext->PSSetSamplers(0, 1, &m_sampleState);

			for (auto mesh : object->GetMeshes())
			{
				container->GetMesh(mesh)->SendData(deviceContext);
				m_indexCount = container->GetMesh(mesh)->GetIndexCount();
				m_renderType = container->GetMesh(mesh)->GetRenderType();


				BaseShader::Render(deviceContext, d3d, resourceContainer);
			}
		}
	}

	ClearPipeline(deviceContext);
}