#include "PrePassFinalShader.h"
#include "../DXFramework/ResourceContainer.h"

PrePassFinalShader::PrePassFinalShader(ID3D11Device* device, HWND hwnd, bool msaa) : BaseShader(device, hwnd)
{
	if (msaa)
	{
		InitShader(L"../res/shaders/prepass_final_vs.hlsl", L"../res/shaders/prepass_final_msaa_ps.hlsl");
	}
	else
	{
		InitShader(L"../res/shaders/prepass_final_vs.hlsl", L"../res/shaders/prepass_final_ps.hlsl");
	}
}

PrePassFinalShader::~PrePassFinalShader()
{
	if (m_matrixBuffer)
	{
		m_matrixBuffer->Release();
		m_matrixBuffer = nullptr;
	}

	if (m_sampleState)
	{
		m_sampleState->Release();
		m_sampleState = nullptr;
	}

	if (m_materialBuffer)
	{
		m_materialBuffer->Release();
		m_materialBuffer = nullptr;
	}

	if (m_layout)
	{
		m_layout->Release();
		m_layout = nullptr;
	}

	BaseShader::~BaseShader();
}

void PrePassFinalShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	HRESULT result;
	D3D11_SAMPLER_DESC samplerDesc;
	D3D11_BLEND_DESC blendDesc;
	D3D11_DEPTH_STENCIL_DESC depthStencilDesc;

	loadVertexShader(vsFilename);

	loadPixelShader(psFilename);

	// Create a texture sampler state description.
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 16;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// create the depth enabled stencil state
	m_depthStencilState = D3D::instance()->CreateDefaultDepthStencilState();

	// create the alpha disabled blend state
	m_blendState = D3D::instance()->CreateDefaultBlendState();
	m_rasterizerState = D3D::instance()->CreateDefaultRasterizerState();

	// Create the texture sampler state.
	result = m_device->CreateSamplerState(&samplerDesc, &m_sampleState);
	if (FAILED(result))
	{
		printf("Sample State creation failed.");
	}

	//Create the constant buffer to access the vertex shader constant buffer
	m_matrixBuffer = D3D::instance()->CreateConstantBuffer(sizeof(MatrixBufferType), true, true, NULL);

	//Create the material constant buffer
	m_materialBuffer = D3D::instance()->CreateConstantBuffer(sizeof(MaterialBufferType), true, true, NULL);
}

void PrePassFinalShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	HRESULT result;
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	MatrixBufferType* dataPtr;
	MaterialBufferType* matPtr;
	unsigned int bufferNumber;
	XMMATRIX tworld, tview, tproj;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	// Reset to the default view
	d3d->SetIntermediateBufferRenderTarget();
	d3d->ResetViewport();

	container->GetCamera(m_renderCameras[0])->GetViewMatrix(tview);
	tview = XMMatrixTranspose(tview);

	container->GetCamera(m_renderCameras[0])->GetProjectionMatrix(tproj);
	tproj = XMMatrixTranspose(tproj);

	// Set the sampler state in the pixel shader.
	deviceContext->PSSetSamplers(0, 1, &m_sampleState);

	// Set light buffer texture in the pixel shader.
	ID3D11ShaderResourceView* lightingTexture = container->GetBuffers(m_renderBuffers[0])->GetShaderResourceView(0);
	deviceContext->PSSetShaderResources(1, 1, &lightingTexture);

	for (auto object : m_renderObjects)
	{
		tworld = object->GetTransformMatrix();
		tworld = XMMatrixTranspose(tworld);

		// Lock the constant buffer so it can be written to.
		result = deviceContext->Map(m_matrixBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);

		// Get a pointer to the data in the constant buffer.
		dataPtr = (MatrixBufferType*)mappedResource.pData;

		// Copy the matrices into the constant buffer.
		dataPtr->world = tworld;// worldMatrix;
		dataPtr->view = tview;
		dataPtr->projection = tproj;

		// Unlock the constant buffer.
		deviceContext->Unmap(m_matrixBuffer, 0);

		// Set the position of the constant buffer in the vertex shader.
		bufferNumber = 0;

		// Now set the constant buffer in the vertex shader with the updated values.
		deviceContext->VSSetConstantBuffers(bufferNumber, 1, &m_matrixBuffer);

		// Send material data to pixel shader
		deviceContext->Map(m_materialBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);

		//Get a pointer to the constant buffer data
		matPtr = (MaterialBufferType*)mappedResource.pData;

		//Copy the material data into the constant buffer
		matPtr->specularAlbedo = object->GetSpecularAlbedo();
		matPtr->padding = 0.0f;

		//Unlcok the constant buffer
		deviceContext->Unmap(m_materialBuffer, 0);

		bufferNumber = 0;

		//Set the constant buffer in the pixel shader
		deviceContext->PSSetConstantBuffers(bufferNumber, 1, &m_materialBuffer);

		// Set the object diffuse texture in the pixel shader
		ID3D11ShaderResourceView* diffuseTexture = container->GetTexture(object->GetTextureIndex(0))->GetTexture();
		deviceContext->PSSetShaderResources(0, 1, &diffuseTexture);

		// Render each of the meshes present in this object
		std::vector<int> meshes = object->GetMeshes();

		for (int i = 0; i < meshes.size(); ++i)
		{
			container->GetMesh(meshes[i])->SendData(deviceContext);
			m_renderType = container->GetMesh(meshes[i])->GetRenderType();
			m_indexCount = container->GetMesh(meshes[i])->GetIndexCount();
			BaseShader::Render(deviceContext, d3d, resourceContainer);
		}
	}

	ClearPipeline(deviceContext);
}