#include "PrePassGBufferShader.h"
#include "../DXFramework/ResourceContainer.h"

PrePassGBufferShader::PrePassGBufferShader(ID3D11Device* device, HWND hwnd) : BaseShader(device, hwnd)
{
	InitShader(L"../res/shaders/prepass_g_vs.hlsl", L"../res/shaders/prepass_g_ps.hlsl");
}

PrePassGBufferShader::~PrePassGBufferShader()
{
	if (m_matrixBuffer)
	{
		m_matrixBuffer->Release();
		m_matrixBuffer = nullptr;
	}

	if (m_sampleState)
	{
		m_sampleState->Release();
		m_sampleState = nullptr;
	}

	if (m_materialBuffer)
	{
		m_materialBuffer->Release();
		m_materialBuffer = nullptr;
	}

	if (m_layout)
	{
		m_layout->Release();
		m_layout = nullptr;
	}

	BaseShader::~BaseShader();

}
void PrePassGBufferShader::InitShader(WCHAR* vsFilename, WCHAR* psFilename)
{
	D3D11_SAMPLER_DESC samplerDesc;

	loadVertexShader(vsFilename);
	loadPixelShader(psFilename);

	//Create the constant buffer to access the vertex shader constant buffer
	m_matrixBuffer = D3D::instance()->CreateConstantBuffer(sizeof(MatrixBufferType), true, true, NULL);

	//Create the material constant buffer
	m_materialBuffer = D3D::instance()->CreateConstantBuffer(sizeof(MaterialBufferType), true, true, NULL);

	// Create a texture sampler state description.
	samplerDesc.Filter = D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	samplerDesc.AddressU = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressV = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.AddressW = D3D11_TEXTURE_ADDRESS_WRAP;
	samplerDesc.MipLODBias = 0.0f;
	samplerDesc.MaxAnisotropy = 16;
	samplerDesc.ComparisonFunc = D3D11_COMPARISON_ALWAYS;
	samplerDesc.BorderColor[0] = 0;
	samplerDesc.BorderColor[1] = 0;
	samplerDesc.BorderColor[2] = 0;
	samplerDesc.BorderColor[3] = 0;
	samplerDesc.MinLOD = 0;
	samplerDesc.MaxLOD = D3D11_FLOAT32_MAX;

	// Create the texture sampler state.
	m_device->CreateSamplerState(&samplerDesc, &m_sampleState);
	
	// create the depth enabled stencil state
	m_depthStencilState = D3D::instance()->CreateDefaultDepthStencilState();

	// create the alpha disabled blend state
	m_blendState = D3D::instance()->CreateDefaultBlendState();
	m_rasterizerState = D3D::instance()->CreateDefaultRasterizerState();
}

void PrePassGBufferShader::Render(ID3D11DeviceContext* deviceContext, D3D* d3d, void* resourceContainer)
{
	HRESULT result;
	D3D11_MAPPED_SUBRESOURCE mappedResource;
	MatrixBufferType* dataPtr;
	MaterialBufferType* matPtr;
	unsigned int bufferNumber;
	XMMATRIX tworld, tview, tproj;

	ResourceContainer* container = reinterpret_cast<ResourceContainer*>(resourceContainer);

	ConfigurePipeline(deviceContext);

	// Set the sampler state in the pixel shader.
	deviceContext->PSSetSamplers(0, 1, &m_sampleState);

	// Set the g-buffer as the render target
	container->GetBuffers(m_renderBuffers[0])->SetRenderTargets();
	container->GetBuffers(m_renderBuffers[0])->ClearRenderTargets(0.0f, 0.0f, 0.0f, 1.0f);

	container->GetCamera(m_renderCameras[0])->GetViewMatrix(tview);
	tview = XMMatrixTranspose(tview);

	container->GetCamera(m_renderCameras[0])->GetProjectionMatrix(tproj);
	tproj = XMMatrixTranspose(tproj);

	for (auto object : m_renderObjects)
	{
		std::vector<int> meshes = object->GetMeshes();

		tworld = object->GetTransformMatrix();
		tworld = XMMatrixTranspose(tworld);

		// Lock the constant buffer so it can be written to.
		result = deviceContext->Map(m_matrixBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);

		// Get a pointer to the data in the constant buffer.
		dataPtr = (MatrixBufferType*)mappedResource.pData;

		// Copy the matrices into the constant buffer.
		dataPtr->world = tworld;// worldMatrix;
		dataPtr->view = tview;
		dataPtr->projection = tproj;

		// Unlock the constant buffer.
		deviceContext->Unmap(m_matrixBuffer, 0);

		// Set the position of the constant buffer in the vertex shader.
		bufferNumber = 0;

		// Now set the constant buffer in the vertex shader with the updated values.
		deviceContext->VSSetConstantBuffers(bufferNumber, 1, &m_matrixBuffer);

		// Send material data to pixel shader
		deviceContext->Map(m_materialBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedResource);

		//Get a pointer to the constant buffer data
		matPtr = (MaterialBufferType*)mappedResource.pData;

		//Copy the material data into the constant buffer
		matPtr->padding = XMFLOAT3(1.0f, 1.0f, 1.0f);
		matPtr->specularPower = object->GetSpecularPower();

		//Unlcok the constant buffer
		deviceContext->Unmap(m_materialBuffer, 0);

		bufferNumber = 0;

		//Set the constant buffer in the pixel shader
		deviceContext->PSSetConstantBuffers(bufferNumber, 1, &m_materialBuffer);

		ID3D11ShaderResourceView* texture = container->GetTexture(object->GetTextureIndex(1))->GetTexture();
		// Set shader texture resource in the pixel shader.
		deviceContext->PSSetShaderResources(0, 1, &texture);

		for (int i = 0; i < meshes.size(); ++i)
		{
			container->GetMesh(meshes[i])->SendData(deviceContext);
			m_renderType = container->GetMesh(meshes[i])->GetRenderType();
			m_indexCount = container->GetMesh(meshes[i])->GetIndexCount();
			BaseShader::Render(deviceContext, d3d, resourceContainer);
		}
	}

	ClearPipeline(deviceContext);
}