#include "SceneObject.h"

SceneObject::SceneObject()
{
	m_specularAlbedo = XMFLOAT3(0.0f, 0.0f, 0.0f);
	m_specularPower = 0.0f;
	m_position = XMFLOAT3(0.0f, 0.0f, 0.0f);
	m_rotation = XMFLOAT3(0.0f, 0.0f, 0.0f);
	m_scale = XMFLOAT3(1.0f, 1.0f, 1.0f);
	m_transformMatrix = XMMatrixIdentity();
	m_transformUpdate = true;
}

SceneObject::SceneObject(SceneObject& copy)
{
	m_position = copy.m_position;
	m_rotation = copy.m_rotation;
	m_scale = copy.m_scale;
	m_meshes = copy.m_meshes;
	m_textures = copy.m_textures;
}

SceneObject::~SceneObject()
{
}

void SceneObject::AddMesh(int meshIndex)
{
	m_meshes.push_back(meshIndex);
}

void SceneObject::RemoveMesh(int mesh)
{
	m_meshes.erase(m_meshes.begin() + mesh);
}

void SceneObject::RemoveMeshes(int meshIndex)
{
	auto mesh = m_meshes.begin();
	while (mesh != m_meshes.end())
	{
		if (*mesh == meshIndex)
		{
			m_meshes.erase(mesh);
		}
		else
		{
			mesh++;
		}
	}
}

void SceneObject::AddTexture(int textureIndex)
{
	m_textures.push_back(textureIndex);
}

void SceneObject::RemoveTexture(int texture)
{
	m_textures.erase(m_textures.begin() + texture);
}

void SceneObject::RemoveTextures(int textureIndex)
{
	auto texture = m_textures.begin();
	while (texture != m_textures.end())
	{
		if (*texture != textureIndex)
		{
			m_textures.erase(texture);
		}
		else
		{
			texture++;
		}
	}
}

void SceneObject::AttachToShader(int shaderIndex)
{
	m_renderShaders.push_back(shaderIndex);
}

void SceneObject::DetachFromShader(int shaderIndex)
{
	auto shader = m_renderShaders.begin();
	while (shader != m_renderShaders.end())
	{
		if (*shader == shaderIndex)
		{
			m_renderShaders.erase(shader);
		}
		else
		{
			shader++;
		}
	}
}

std::vector<int>& SceneObject::GetMeshes()
{
	return m_meshes;
}

int SceneObject::GetMeshIndex(int mesh)
{
	if (mesh >= m_meshes.size() || mesh < 0)
		return -1;

	return m_meshes[mesh];
}

int SceneObject::GetTextureIndex(int texture)
{
	if (texture >= m_textures.size() || texture < 0)
		return -1;

	return m_textures[texture];
}

void SceneObject::SetPosition(XMFLOAT3 position)
{
	m_position = position;
	m_transformUpdate = true;
}

void SceneObject::SetRotation(XMFLOAT3 rotation)
{
	m_rotation = rotation;
	m_transformUpdate = true;
}

void SceneObject::SetScale(XMFLOAT3 scale)
{
	m_scale = scale;
}

XMFLOAT3 SceneObject::GetPosition()
{
	return m_position;
}

void SceneObject::GetPosition(float& x, float& y, float& z)
{
	x = m_position.x;
	y = m_position.y;
	z = m_position.z;
}

XMFLOAT3 SceneObject::GetRotation()
{
	return m_rotation;
}

void SceneObject::GetRotation(float& x, float& y, float& z)
{
	x = m_rotation.x;
	y = m_rotation.y;
	z = m_rotation.z;
}

XMFLOAT3 SceneObject::GetScale()
{
	return m_scale;
}

void SceneObject::GetScale(float& x, float& y, float& z)
{
	x = m_scale.x;
	y = m_scale.y;
	z = m_scale.z;
}

XMMATRIX SceneObject::GetTransformMatrix()
{
	if (m_transformUpdate)
		RecalculateTransformMatrix();
	return m_transformMatrix;
}

void SceneObject::RecalculateTransformMatrix()
{
	XMMATRIX transform, rotation, scale;
	
	transform = XMMatrixTranslation(m_position.x, m_position.y, m_position.z);
	rotation = XMMatrixRotationRollPitchYaw(m_rotation.x, m_rotation.y, m_rotation.z);
	scale = XMMatrixScaling(m_scale.x, m_scale.y, m_scale.z);

	transform = XMMatrixMultiply(XMMatrixMultiply(transform, rotation), scale);
	m_transformMatrix = transform;
}

void SceneObject::SetName(const std::wstring& name)
{
	m_objectName = name;
}

std::wstring SceneObject::GetName()
{
	return m_objectName;
}

void SceneObject::SetSpecularAlbedo(float r, float g, float b)
{
	m_specularAlbedo = XMFLOAT3(r, g, b);
}

XMFLOAT3 SceneObject::GetSpecularAlbedo()
{
	return m_specularAlbedo;
}

void SceneObject::SetSpecularPower(float pow)
{
	m_specularPower = pow;
}

float SceneObject::GetSpecularPower()
{
	return m_specularPower;
}