#include "System.h"

System::System(BaseApplication* application)
{
	int screenWidth, screenHeight;
	int resolutionWidth, resolutionHeight;

	// init screen width/height to zero
	screenWidth = 0;
	screenHeight = 0;

	resolutionWidth = 0;
	resolutionHeight = 0;

	// init windows api
	InitializeWindows(screenWidth, screenHeight);

	// init the app object
	m_Application = application;
	m_Application->init(m_hinstance, m_hwnd, screenWidth, screenHeight, &m_Input, &m_INIReader);
}

System::~System()
{
	// release the app object
	
	// shutdown the window
	ShutdownWindows();
}

void System::Run()
{
	MSG msg;
	bool done, result;

	// init the message structure
	ZeroMemory(&msg, sizeof(MSG));

	// loop the app until there is a quit message
	done = false;
	while (!done)
	{
		// handle windows messages
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		if (msg.message == WM_QUIT)
		{
			done = true;
		}
		else
		{
			// do frame processing
			result = Frame();
			if (!result)
			{
				done = true;
			}
		}
	}

	return;
}

bool System::Frame()
{
	bool result;

	// Frame processing for application object
	result = m_Application->Frame();
	if (!result)
	{
		return false;
	}

	return true;
}

LRESULT CALLBACK System::MessageHandler(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
	return DefWindowProc(hwnd, umsg, wparam, lparam);
}

bool System::InitializeWindows(int& screenWidth, int& screenHeight)
{
	WNDCLASSEX wc;
	DEVMODE dmScreenSettings;
	int posX, posY;

	// point the ini reader to the settings file
	m_INIReader.setFilename("../res/data/settings.ini");

	// set the global pointer to the system object
	ApplicationHandle = this;

	// get the instance of this application
	m_hinstance = GetModuleHandle(NULL);

	// give the app a name
	m_appName = "Engine";

	// setup the windows class
	wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc = WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = m_hinstance;
	wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);
	wc.hIconSm = wc.hIcon;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = m_appName;
	wc.cbSize = sizeof(WNDCLASSEX);

	// register window class
	RegisterClassEx(&wc);

	// determine resolution of desktop screen
	screenWidth = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);

	// find the settings resolution
	int settingsWidth, settingsHeight;
	settingsWidth = std::stoi(m_INIReader.getValue("resolutionX", "settings"));
	settingsHeight = std::stoi(m_INIReader.getValue("resolutionY", "settings"));

	string fullscreen = m_INIReader.getValue("fullscreen", "settings");

	m_fullscreen = (fullscreen == "enabled");

	// setup the screen settings depending on whether it is running in full screen or in windowed mode
	if (m_fullscreen)
	{
		// set the screen to max size of the user's desktop
		memset(&dmScreenSettings, 0, sizeof(dmScreenSettings));
		dmScreenSettings.dmSize = sizeof(dmScreenSettings);
		dmScreenSettings.dmPelsWidth = (unsigned long)screenWidth;
		dmScreenSettings.dmPelsHeight = (unsigned long)screenHeight;
		dmScreenSettings.dmBitsPerPel = 32;
		dmScreenSettings.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;

		// change display settings to full screen
		ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN);
	
		//set window position to top left of screen
		posX = posY = 0;
	}
	else
	{
		// check that the window resolution settings are not larger than the users screen size
		if (settingsWidth < screenWidth || settingsHeight < screenHeight)
		{
			screenWidth = settingsWidth;
			screenHeight = settingsHeight;
		}
		else
		{
			m_INIReader.setValue(std::to_string(screenWidth), "resolutionX", "settings");
			m_INIReader.setValue(std::to_string(screenHeight), "resolutionY", "settings");
		}

		// place the window in the middle of the screen
		posX = (GetSystemMetrics(SM_CXSCREEN) - screenWidth) / 2;
		posY = (GetSystemMetrics(SM_CYSCREEN) - screenHeight) / 2;
	}

	// create the window with the screen settings and get the handle
	m_hwnd = CreateWindowEx(WS_EX_APPWINDOW, m_appName, m_appName,
		WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_POPUP,
		posX, posY, screenWidth, screenHeight, NULL, NULL, m_hinstance, NULL);

	// set the window as the main focus
	ShowWindow(m_hwnd, SW_SHOW);
	SetForegroundWindow(m_hwnd);
	SetFocus(m_hwnd);

	// hide the mouse cursor
	ShowCursor(false);

	return true;
}

void System::ShutdownWindows()
{
	// show the mouse cursor
	ShowCursor(true);

	// fix the display settings if leaving full screen mode
	if (m_fullscreen)
	{
		ChangeDisplaySettings(NULL, 0);
	}

	// remove the window
	DestroyWindow(m_hwnd);
	m_hwnd = NULL;

	// remove the application instance
	UnregisterClass(m_appName, m_hinstance);
	m_hinstance = NULL;

	// release the pointer to this class
	ApplicationHandle = NULL;
}

LRESULT CALLBACK System::WndProc(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
	switch (umsg)
	{
		// Handle keyboard and mouse input.
	case WM_KEYDOWN:
	{
		ApplicationHandle->m_Input.SetKeyDown(wparam);
		break;
	}
	case WM_KEYUP:
	{
		ApplicationHandle->m_Input.SetKeyUp(wparam);
		break;
	}
	case WM_MOUSEMOVE:
	{
		ApplicationHandle->m_Input.setMouseX(LOWORD(lparam));
		ApplicationHandle->m_Input.setMouseY(HIWORD(lparam));
		break;
	}
	case WM_LBUTTONDOWN:
	{
		ApplicationHandle->m_Input.setLeftMouse(true);
		break;
	}
	case WM_RBUTTONDOWN:
	{
		ApplicationHandle->m_Input.setRightMouse(true);
		break;
	}
	case WM_LBUTTONUP:
	{
		ApplicationHandle->m_Input.setLeftMouse(false);
		break;
	}
	case WM_RBUTTONUP:
	{
		ApplicationHandle->m_Input.setRightMouse(false);
		break;
	}
	// Check if the window is being destroyed.
	case WM_DESTROY:
	{
		PostQuitMessage(0);
		return 0;
	}

	// Check if the window is being closed.
	case WM_CLOSE:
	{
		PostQuitMessage(0);
		return 0;
	}

	}
	return ApplicationHandle->MessageHandler(hwnd, umsg, wparam, lparam);
}