#include "TestScene.h"

TestScene::TestScene(D3D* d3d, HWND hwnd, INIReader* iniReader) : BaseScene(d3d, hwnd, iniReader)
{
	BaseShader* shader;
	BaseMesh* mesh;
	DeferredBuffers* buffers;
	Camera* camera;
	Light* light;

	// Create the main camera using the app camera settings
	iniReader->setFilename("../res/data/settings.ini");

	float fov = stof(iniReader->getValue("fov", "settings"));
	camera = new Camera(fov, d3d->GetResolutionWidth(), d3d->GetResolutionHeight(), 1.0f, 1000.0f);
	camera->SetPosition(0.0f, 0.0f, -10.0f);
	m_resourceContainer->AddCamera(L"MAIN_CAMERA", camera);

	string dofSetting = iniReader->getValue("dofSetting", "settings");

	if (dofSetting == "disabled")
		m_dofSetting = DoFSetting::DISABLED;
	else if (dofSetting == "gaussian")
		m_dofSetting = DoFSetting::GAUSSIAN;
	else if (dofSetting == "advanced_gaussian")
		m_dofSetting = DoFSetting::GAUSSIAN_DEPTH;
	else if (dofSetting == "bokeh")
		m_dofSetting = DoFSetting::BOKEH;

	// Create the sphere mesh and brick texture
	mesh = new SphereMesh(m_device, m_deviceContext, NULL);
	Texture* brick_texture = new Texture(m_device, m_deviceContext, L"../res/assets/textures/stone01.dds");
	Texture* bump_texture = new Texture(m_device, m_deviceContext, L"../res/assets/textures/bump01.dds");
	m_resourceContainer->AddMesh(L"SPHERE_MESH", mesh);
	m_resourceContainer->AddTexture(L"stone01.dds", brick_texture);
	m_resourceContainer->AddTexture(L"bump01.dds", bump_texture);

	// Create the test sphere object
	SceneObject* test_sphere = new SceneObject;
	test_sphere->AddMesh(m_resourceContainer->GetMeshIndex(L"SPHERE_MESH"));
	test_sphere->AddTexture(m_resourceContainer->GetTextureIndex(L"stone01.dds"));
	test_sphere->AddTexture(m_resourceContainer->GetTextureIndex(L"bump01.dds"));
	test_sphere->SetName(L"SPHERE");
	test_sphere->SetPosition(XMFLOAT3(0.0f, 0.0f, 0.0f));
	test_sphere->SetSpecularAlbedo(0.4f, 0.4, 0.4f);
	test_sphere->SetSpecularPower(25.0f);
	m_objects.push_back(test_sphere);

	// Create a second test sphere object
	SceneObject* test_sphere_2 = new SceneObject;
	test_sphere_2->AddMesh(m_resourceContainer->GetMeshIndex(L"SPHERE_MESH"));
	test_sphere_2->AddTexture(m_resourceContainer->GetTextureIndex(L"stone01.dds"));
	test_sphere_2->AddTexture(m_resourceContainer->GetTextureIndex(L"bump01.dds"));
	test_sphere_2->SetName(L"SPHERE_02");
	test_sphere_2->SetPosition(XMFLOAT3(3.0f, 0.0f, 0.0f));
	test_sphere_2->SetSpecularAlbedo(0.4f, 0.4, 0.4f);
	test_sphere_2->SetSpecularPower(25.0f);
	m_objects.push_back(test_sphere_2);

	// Create the render window mesh
	mesh = 0;
	mesh = new QuadMesh(m_device, m_deviceContext, NULL);

	// Create the render window object
	SceneObject* render_window = new SceneObject;
	render_window->AddMesh(m_resourceContainer->AddMesh(L"RENDER_WINDOW", mesh));
	render_window->SetName(L"RENDER_WINDOW");
	m_objects.push_back(render_window);

	switch (d3d->GetAntiAliasingSetting())
	{
	case AntiAliasingSetting::DISABLED:
	{
		InitRendererNoAA();
	}
	break;
	case AntiAliasingSetting::MSAA2X:
	{
		InitRendererAA(2);
	}
	break;
	case AntiAliasingSetting::MSAA4X:
	{
		InitRendererAA(4);
	}
	break;
	case AntiAliasingSetting::MSAA8X:
	{
		InitRendererAA(8);
	}
	break;
	case AntiAliasingSetting::MSAA16X:
	{
		InitRendererAA(16);
	}
	break;
	case AntiAliasingSetting::FXAA:
	{
		InitRendererNoAA();
		InitFXAA();
	}
	break;
	case AntiAliasingSetting::TXAA:
	{
		InitRendererNoAA();
	}
	break;
	default:
	{
		InitRendererNoAA();
	}
	break;
	}
	
	InitDepthShader();
	InitShadowMapShader();
	InitAmbientOcclusion();

	if (m_dofSetting == DoFSetting::BOKEH)
		InitDiskDoF(8);


	// Create an ambient light
	light = 0;
	light = new Light;
	light->SetDirection(0.0f, 0.0f, 0.0f);
	light->SetDiffuseColour(0.4f, 0.4f, 0.4f, 1.0f);
	light->SetPosition(0.0f, 0.0f, 0.0f);
	light->SetConstantAttenuation(1.0f);
	light->SetLinearAttenuation(0.125f);
	light->SetQuadraticAttenuation(0.0f);
	light->SetLightType(0);
	m_resourceContainer->AddLight(L"AMBIENT_LIGHT", light);

	// Create a simple directional light
	light = 0;
	light = new Light;
	light->SetDirection(1.0f, 0.0f, 0.0f);
	light->SetDiffuseColour(1.0f, 1.0f, 1.0f, 1.0f);
	light->SetPosition(-16.0f, 5.0f, 0.0f);
	light->SetConstantAttenuation(1.0f);
	light->SetLinearAttenuation(0.125f);
	light->SetQuadraticAttenuation(0.0f);
	light->SetLightType(1);
	light->GenerateShadowMapBuffer(m_device, m_deviceContext, 1024, 1024, 1, 1);
	light->GenerateViewMatrix();
	light->GenerateProjectionMatrix(camera->GetNearClip(), camera->GetFarClip());
	m_resourceContainer->AddLight(L"DIRECTIONAL_LIGHT", light);

	// Add the correct rendering objects to the shaders
	m_resourceContainer->GetShader(L"G_BUFFER_SHADER")->AddObject(test_sphere);
	m_resourceContainer->GetShader(L"G_BUFFER_SHADER")->AddObject(test_sphere_2);
	m_resourceContainer->GetShader(L"MASK_SHADER")->AddObject(render_window);
	m_resourceContainer->GetShader(L"LIGHT_SHADER")->AddObject(render_window);
	m_resourceContainer->GetShader(L"FINAL_SHADER")->AddObject(test_sphere);
	m_resourceContainer->GetShader(L"DEPTH_SHADER")->AddObject(test_sphere);
	m_resourceContainer->GetShader(L"SHADOW_MAP_SHADER")->AddObject(test_sphere);
	m_resourceContainer->GetShader(L"FINAL_SHADER")->AddObject(test_sphere_2);
	m_resourceContainer->GetShader(L"DEPTH_SHADER")->AddObject(test_sphere_2);
	m_resourceContainer->GetShader(L"SHADOW_MAP_SHADER")->AddObject(test_sphere_2);
	m_resourceContainer->GetShader(L"SHADOW_MAP_SHADER")->AddLight(m_resourceContainer->GetLightIndex(L"DIRECTIONAL_LIGHT"));
	m_resourceContainer->GetShader(L"LIGHT_SHADER")->AddLight(m_resourceContainer->GetLightIndex(L"AMBIENT_LIGHT"));
	m_resourceContainer->GetShader(L"LIGHT_SHADER")->AddLight(m_resourceContainer->GetLightIndex(L"DIRECTIONAL_LIGHT"));
}

TestScene::~TestScene()
{
}

void TestScene::Update(Input* input, float frameTime)
{
	HandleInput(input, frameTime);

	m_resourceContainer->GetCamera(L"MAIN_CAMERA")->Update();

	if (m_dofSetting == DoFSetting::BOKEH)
	{
		m_resourceContainer->GetFloat(L"DOF_FOCAL_WIDTH")->UpdateParameter(0, 20.0f);
		m_resourceContainer->GetFloat(L"DOF_FOCAL_DISTANCE")->UpdateParameter(0, 7.0f);
		m_resourceContainer->GetFloat(L"DOF_FAR_CLIP")->UpdateParameter(0, m_resourceContainer->GetCamera(L"MAIN_CAMERA")->GetFarClip());
		m_resourceContainer->GetFloat(L"DOF_MAX_COC")->UpdateParameter(0, 10.0f);
	}

	Render();
}

void TestScene::Render()
{
	ID3D11Resource* buffer;

	// Clear the scene
	m_Direct3D->BeginScene(1.0f, 1.0f, 1.0f, 0.0f);

	// Render the depth buffer
	m_resourceContainer->GetShader(L"DEPTH_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);

	// Render the ao buffer
	m_resourceContainer->GetShader(L"AMBIENT_OCCLUSION_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);

	// Apply a bilateral blur to the ao texture
	m_resourceContainer->GetShader(L"BILATERAL_BLUR_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);

	// Generate shadow maps for lights
	m_resourceContainer->GetShader(L"SHADOW_MAP_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);

	// Execute the deferred renderer
	m_resourceContainer->GetShader(L"G_BUFFER_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
	//m_resourceContainer->GetShader(L"MASK_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
	m_resourceContainer->GetShader(L"LIGHT_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
	m_resourceContainer->GetShader(L"FINAL_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);

	// Execute the DoF Post Process
	if (m_dofSetting == DoFSetting::BOKEH)
	{
		m_resourceContainer->GetShader(L"DOF_DISK_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
		m_resourceContainer->GetShader(L"DOF_BUFFER_SCALING_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
	}

	// Execute the Fxaa Post Process
	if (m_Direct3D->GetAntiAliasingSetting() == AntiAliasingSetting::FXAA)
	{
		m_resourceContainer->GetShader(L"FXAA_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
		m_resourceContainer->GetShader(L"FXAA_BUFFER_SCALING_SHADER")->Render(m_deviceContext, m_Direct3D, m_resourceContainer);
	}
	
	// Present the rendered scene to the screen
	m_Direct3D->EndScene();
}

void TestScene::HandleInput(Input* input, float frameTime)
{
	Camera* camera = m_resourceContainer->GetCamera(L"MAIN_CAMERA");

	// Set the frame time for calculating the updated position.
	camera->SetFrameTime(frameTime);

	// Handle the input.
	if (input->isKeyDown('W'))
	{
		// forward
		camera->MoveForward();
	}
	if (input->isKeyDown('S'))
	{
		// back
		camera->MoveBackward();
	}
	if (input->isKeyDown('A'))
	{
		// Strafe Left
		camera->StrafeLeft();
	}
	if (input->isKeyDown('D'))
	{
		// Strafe Right
		camera->StrafeRight();
	}
	if (input->isKeyDown('Q'))
	{
		// Down
		camera->MoveDownward();
	}
	if (input->isKeyDown('E'))
	{
		// Up
		camera->MoveUpward();
	}
	if (input->isKeyDown(VK_UP))
	{
		// rotate up
		camera->TurnUp();
	}
	if (input->isKeyDown(VK_DOWN))
	{
		// rotate down
		camera->TurnDown();
	}
	if (input->isKeyDown(VK_LEFT))
	{
		// rotate left
		camera->TurnLeft();
	}
	if (input->isKeyDown(VK_RIGHT))
	{
		// rotate right
		camera->TurnRight();
	}

	if (input->isKeyDown(VK_RETURN))
	{
		if (m_resourceContainer->GetFloat(L"SHADOW_SWITCH")->GetParameter(0) == 0.0f)
		{
			m_resourceContainer->GetFloat(L"SHADOW_SWITCH")->UpdateParameter(0, 1.0f);
		}
		else
		{
			m_resourceContainer->GetFloat(L"SHADOW_SWITCH")->UpdateParameter(0, 0.0f);
		}
		input->SetKeyUp(VK_RETURN);
	}

	if (input->isKeyDown(VK_TAB))
	{
		CaptureSceneBuffers();
	}
}
