// texture.cpp
#include "texture.h"

Texture::Texture(ID3D11Device* device, ID3D11DeviceContext* deviceContext, WCHAR* filename)
{
	HRESULT result;

	// check if file exists
	if (!filename)
	{
		filename = L"../res/assets/textures/DefaultDiffuse.png";
	}
	// if not set default texture
	if (!does_file_exist(filename))
	{
		// change default texture
		filename = L"../res/assets/textures/DefaultDiffuse.png";
	}

	// check file extension for correct loading function.
	std::wstring fn(filename);
	std::string::size_type idx;
	std::wstring extension;

	idx = fn.rfind('.');

	if (idx != std::string::npos)
	{
		extension = fn.substr(idx + 1);
	}
	else
	{
		// No extension found
	}

	// Load the texture in.
	if (extension == L"dds")
	{
		result = CreateDDSTextureFromFile(device, deviceContext, filename, NULL, &m_texture, 0, NULL);
	}
	else
	{
		result = CreateWICTextureFromFile(device, deviceContext, filename, NULL, &m_texture, 0);
	}
	
	if (FAILED(result))
	{
		MessageBox(NULL, (LPCSTR)"Texture loading error", (LPCSTR)"ERROR", MB_OK);
	}
}

/**
* Create the texture as a texture array
* @param	device	handle for the device used by the current application
* @param	filenames	filenames of all the texture files to be used in the texture array
*/
Texture::Texture(ID3D11Device* device, ID3D11DeviceContext* deviceContext, WCHAR** filenames)
{
	ID3D11ShaderResourceView** textureViews;
	HRESULT result;

	int file_count = sizeof(filenames) / sizeof(WCHAR*);

	textureViews = new ID3D11ShaderResourceView*[file_count];

	for (int i = 0; i < file_count; i++)
	{
		// check if file exists
		if (!filenames[0])
		{
			filenames[i] = L"../res/DefaultDiffuse.png";
		}
		// if not set default texture
		if (!does_file_exist(filenames[i]))
		{
			// change default texture
			filenames[i] = L"../res/DefaultDiffuse.png";
		}


		// check file extension for correct loading function.
		std::wstring fn(filenames[0]);
		std::string::size_type idx;
		std::wstring extension;

		idx = fn.rfind('.');

		if (idx != std::string::npos)
		{
			extension = fn.substr(idx + 1);
		}
		else
		{
			// No extension found
		}

		// Load the texture in.
		if (extension == L"dds")
		{
			result = CreateDDSTextureFromFile(device, deviceContext, filenames[i], NULL, &textureViews[i], 0, NULL);
		}
		else
		{
			result = CreateWICTextureFromFile(device, deviceContext, filenames[i], NULL, &textureViews[i], 0);
		}

		if (FAILED(result))
		{
			MessageBox(NULL, (LPCSTR)"Texture loading error", (LPCSTR)"ERROR", MB_OK);
		}
	}

	ID3D11Texture2D* tex;
	D3D11_TEXTURE2D_DESC desc;
	desc.Width = 2560;
	desc.Height = 1920;
	desc.MipLevels = 0;
	desc.ArraySize = file_count;
	desc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	desc.SampleDesc.Count = 1;
	desc.SampleDesc.Quality = 0;
	desc.Usage = D3D11_USAGE_IMMUTABLE;
	desc.BindFlags = D3D11_BIND_SHADER_RESOURCE;
	desc.CPUAccessFlags = 0;
	desc.MiscFlags = 0;

	device->CreateTexture2D(&desc, NULL, &tex);
	
	for (int i = 0; i < file_count; i++)
	{
		ID3D11Resource* res;
		textureViews[i]->GetResource(&res);
		deviceContext->CopySubresourceRegion(tex, i, 0, 0, 0, res, 0, NULL);
	}

	D3D11_SHADER_RESOURCE_VIEW_DESC viewDesc;
	viewDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	viewDesc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2DARRAY;
	viewDesc.Texture2DArray.MostDetailedMip = 0;
	viewDesc.Texture2DArray.MipLevels = -1;
	viewDesc.Texture2DArray.ArraySize = file_count;
	viewDesc.Texture2DArray.FirstArraySlice = 0;

	device->CreateShaderResourceView(tex, &viewDesc, &m_texture);

	delete[] textureViews;
	textureViews = nullptr;
}

Texture::Texture(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int width, int height, D3D11_SUBRESOURCE_DATA* data)
{
	HRESULT result;
	D3D11_TEXTURE2D_DESC textureDesc;

	textureDesc.Width = width;
	textureDesc.Height = height;
	textureDesc.MipLevels = 1;
	textureDesc.ArraySize = 1;
	textureDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	textureDesc.SampleDesc.Count = 1;
	textureDesc.SampleDesc.Quality = 0;
	textureDesc.Usage = D3D11_USAGE_IMMUTABLE;
	textureDesc.BindFlags = D3D11_BIND_SHADER_RESOURCE;
	textureDesc.CPUAccessFlags = 0;
	textureDesc.MiscFlags = 0;

	ID3D11Texture2D* tex = 0;
	result = device->CreateTexture2D(&textureDesc, data, &tex);

	result = device->CreateShaderResourceView(tex, 0, &m_texture);
}

Texture::~Texture()
{
	// Release the texture resource.
	if (m_texture)
	{
		m_texture->Release();
		m_texture = 0;
	}
}


ID3D11ShaderResourceView* Texture::GetTexture()
{
	return m_texture;
}

bool Texture::does_file_exist(const WCHAR *fname)
{
	std::ifstream infile(fname);
	return infile.good();
}