#include "UnorderedAccessView.h"

// Inititalizes the unordered access view as a read/write texture
void UnorderedAccessView::InitRWTexture(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int width, int height, D3D11_SUBRESOURCE_DATA* initialData)
{
	HRESULT result;
	D3D11_TEXTURE2D_DESC texDesc;
	D3D11_SHADER_RESOURCE_VIEW_DESC srvDesc;
	D3D11_UNORDERED_ACCESS_VIEW_DESC uavDesc;
	ID3D11Texture2D* tex;

	texDesc.Width = width;
	texDesc.Height = height;
	texDesc.ArraySize = 1;
	texDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	texDesc.SampleDesc.Count = 1;
	texDesc.SampleDesc.Quality = 0;
	texDesc.MipLevels = 0;
	texDesc.Usage = D3D11_USAGE_DEFAULT;
	texDesc.BindFlags = (D3D11_BIND_UNORDERED_ACCESS | D3D11_BIND_SHADER_RESOURCE);
	texDesc.CPUAccessFlags = 0;
	texDesc.MiscFlags = 0;

	result = device->CreateTexture2D(&texDesc, initialData, &tex);

	uavDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	uavDesc.ViewDimension = D3D11_UAV_DIMENSION_TEXTURE2D;
	uavDesc.Texture2D.MipSlice = 0;

	result = device->CreateUnorderedAccessView(tex, &uavDesc, &m_UAV);

	srvDesc.Format = DXGI_FORMAT_R32G32B32A32_FLOAT;
	srvDesc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;
	srvDesc.Texture2D.MostDetailedMip = 0;
	srvDesc.Texture2D.MipLevels = 1;

	result = device->CreateShaderResourceView(tex, &srvDesc, &m_SRV);
}

// Initializes the unordered access view as a structured buffer
void UnorderedAccessView::InitStructuredBuffer(ID3D11Device* device, ID3D11DeviceContext* deviceContext, int elementSize, int elementCount, D3D11_SUBRESOURCE_DATA* initialData)
{
	HRESULT result;
	D3D11_BUFFER_DESC bufferDesc;
	D3D11_SHADER_RESOURCE_VIEW_DESC srvDesc;
	D3D11_UNORDERED_ACCESS_VIEW_DESC uavDesc;
	
	bufferDesc.ByteWidth = elementSize * elementCount;
	bufferDesc.BindFlags = (D3D11_BIND_SHADER_RESOURCE | D3D11_BIND_UNORDERED_ACCESS);
	bufferDesc.MiscFlags = D3D11_RESOURCE_MISC_BUFFER_STRUCTURED;
	bufferDesc.StructureByteStride = elementSize;
	bufferDesc.Usage = D3D11_USAGE_DEFAULT;
	bufferDesc.CPUAccessFlags = 0;

	m_structuredBuffer = 0;
	result = device->CreateBuffer(&bufferDesc, initialData, &m_structuredBuffer);

	uavDesc.Format = DXGI_FORMAT_UNKNOWN;
	uavDesc.ViewDimension = D3D11_UAV_DIMENSION_BUFFER;
	uavDesc.Buffer.FirstElement = 0;
	uavDesc.Buffer.NumElements = elementCount;
	uavDesc.Buffer.Flags = 0;

	result = device->CreateUnorderedAccessView(m_structuredBuffer, &uavDesc, &m_UAV);

	srvDesc.Format = DXGI_FORMAT_UNKNOWN;
	srvDesc.ViewDimension = D3D11_SRV_DIMENSION_BUFFEREX;
	srvDesc.BufferEx.FirstElement = 0;
	srvDesc.BufferEx.Flags = 0;
	srvDesc.BufferEx.NumElements = elementCount;

	result = device->CreateShaderResourceView(m_structuredBuffer, &srvDesc, &m_SRV);
}

UnorderedAccessView::UnorderedAccessView()
{
	m_UAV = 0;
	m_SRV = 0;
	m_structuredBuffer = 0;
}

UnorderedAccessView::~UnorderedAccessView()
{
	if (m_UAV)
	{
		m_UAV->Release();
		m_UAV = 0;
	}

	if (m_SRV)
	{
		m_SRV->Release();
		m_SRV = 0;
	}

	if (m_structuredBuffer)
	{
		m_structuredBuffer->Release();
		m_structuredBuffer = 0;
	}
}

ID3D11UnorderedAccessView* UnorderedAccessView::GetUAV()
{
	return m_UAV;
}

ID3D11ShaderResourceView* UnorderedAccessView::GetSRV()
{
	return m_SRV;
}