#include "../res/shaders/shader_settings.hlsli"

// Textures
#if SAMPLE_COUNT > 1
Texture2DMS<float4, SAMPLE_COUNT> sourceTexture : register(t0);
Texture2D depthTexture : register(t1);
#else
Texture2D sourceTexture : register(t0);
Texture2D depthTexture : register(t1);
#endif

// defines
static const int NUM_DOF_TAPS = 12;

// I/O Structs
struct VertexOutput
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};

cbuffer BlurFactors : register(b0)
{
	float focalWidth;
	float focalDistance;
	float farClip;
	float maxCoC;
	//float2 filterTaps[NUM_DOF_TAPS];
}

float GetBlurFactor(float depth)
{
	return smoothstep(0, focalWidth, abs(focalDistance - (depth * farClip)));
}

#if SAMPLE_COUNT > 1
float4 main(VertexOutput input, in uint subSampleIndex : SV_SampleIndex) : SV_TARGET
{
	float4 position = input.pos;

	float width = 0.5f / 1366.0f;
	float height = 0.5f / 768.0f;
	float2 filterTaps[NUM_DOF_TAPS] =
	{
		float2(-0.326212f * width, -0.40581f * height),
		float2(-0.840144f * width, -0.07358f * height),
		float2(-0.840144f * width, 0.457137f * height),
		float2(-0.203345f * width, 0.620716f * height),
		float2(0.96234f * width, -0.194983f * height),
		float2(0.473434f * width, -0.480026f * height),
		float2(0.519456f * width, 0.767022f * height),
		float2(0.185461f * width, -0.893124f * height),
		float2(0.507431f * width, 0.064425f * height),
		float2(0.89642f * width, 0.412458f * height),
		float2(-0.32194f * width, -0.932615f * height),
		float2(-0.791559f * width, -0.59771f * height)
	};

	int3 sampleIndices = int3(position.xy, 0);

	// get center sample color
	float4 colorSum = sourceTexture.Load(sampleIndices.xy, subSampleIndex);
	float totalContribution = 1.0f;

	// get depth and blurriness values for center sample
	float centerDepth = depthTexture.Load(sampleIndices).w;
	float centerBlur = GetBlurFactor(centerDepth);

	if (centerBlur > 0)
	{
		// compute circle of confusion based on blurriness
		float sizeCoC = centerBlur * maxCoC;

		// run through all filter taps
		for (int i = 0; i < NUM_DOF_TAPS; i++)
		{
			// compute sample coordinates
			int2 tapCoord = position.xy + filterTaps[i] * sizeCoC;

			// fetch filter tap sample
			float4 tapColor = sourceTexture.Load(tapCoord.xy, subSampleIndex);
			float tapDepth = depthTexture.Load(int3(tapCoord.xy, 0)).w;
			float tapBlur = GetBlurFactor(tapDepth);

			// compute tap contribution based on depth and blurriness
			float tapContribution = (tapDepth > centerDepth) ? 1.0f : tapBlur;

			// accumulate color and sample contribution
			colorSum += tapColor * tapContribution;
			totalContribution += tapContribution;
		}
	}

	// normalize color sum
	float4 finalColor = colorSum / totalContribution;

	//return float4(totalContribution / (float)NUM_DOF_TAPS, totalContribution / (float)NUM_DOF_TAPS, totalContribution / (float)NUM_DOF_TAPS, 1.0f);
	
	//return colorSum;

	return finalColor;
}
#else
float4 main(float4 position : SV_Position) : SV_TARGET
{
	float width = 0.5f / 1366.0f;
	float height = 0.5f / 768.0f;
	float2 filterTaps[NUM_DOF_TAPS] =
	{
		float2(-0.326212f * width, -0.40581f * height),
		float2(-0.840144f * width, -0.07358f * height),
		float2(-0.840144f * width, 0.457137f * height),
		float2(-0.203345f * width, 0.620716f * height),
		float2(0.96234f * width, -0.194983f * height),
		float2(0.473434f * width, -0.480026f * height),
		float2(0.519456f * width, 0.767022f * height),
		float2(0.185461f * width, -0.893124f * height),
		float2(0.507431f * width, 0.064425f * height),
		float2(0.89642f * width, 0.412458f * height),
		float2(-0.32194f * width, -0.932615f * height),
		float2(-0.791559f * width, -0.59771f * height)
	};

	int3 sampleIndices = int3(position.xy, 0);

	// get center sample color
	float4 colorSum = sourceTexture.Load(sampleIndices);
	float totalContribution = 1.0f;

	// get depth and blurriness values for center sample
	float centerDepth = depthTexture.Load(sampleIndices).w;
	float centerBlur = GetBlurFactor(centerDepth);

	if (centerBlur > 0)
	{
		// compute circle of confusion based on blurriness
		float sizeCoC = centerBlur * maxCoC;

		// run through all filter taps
		for (int i = 0; i < NUM_DOF_TAPS; i++)
		{
			// compute sample coordinates
			float2 tapCoord = position.xy + filterTaps[i] * sizeCoC;

			// fetch filter tap sample
			float4 tapColor = sourceTexture.Load(int3(tapCoord, 0));
			float tapDepth = depthTexture.Load(int3(tapCoord, 0)).w;
			float tapBlur = GetBlurFactor(tapDepth);

			// compute tap contribution based on depth and blurriness
			float tapContribution = (tapDepth > centerDepth) ? 1.0f : tapBlur;

			// accumulate color and sample contribution
			colorSum += tapColor * tapContribution;
			totalContribution += tapContribution;
		}
	}

	// normalize color sum
	float4 finalColor = colorSum / totalContribution;

	return finalColor;
}
#endif