#include "../res/shaders/shader_settings.hlsli"
//Textures
Texture2D diffuseMap : register(t0);
Texture2DMS<float4, SAMPLE_COUNT> lightTexture : register(t1);

SamplerState AnisoSampler : register(s0);

//constants
cbuffer MaterialProperties
{
	float3 specularAlbedo;
	float padding;
};

//input structures
struct PSInput
{
	float4 screenPos : SV_Position;
	float2 texCoord : TEXCOORD0;
};

float4 main(in PSInput input, in uint coverageMask : SV_Coverage) : SV_Target0
{
	//sample the diffuse map
	float3 diffuseAlbedo = diffuseMap.Sample(AnisoSampler, input.texCoord).rgb;

	//determine the indices for the texture based on the current screen position
	int2 sampleIndices = int2(input.screenPos.xy);

	float3 diffuse = 0;
	float specular = 0.0f;
	float numSamplesApplied = 0.0f;

	//loop the MSAA samples and modulate diffuse and specular lighting by albedo
	for (uint i = 0; i < SAMPLE_COUNT; ++i)
	{
		//only apply lighting if the geometry passed the coverage test for the msaa sample point
		if ((coverageMask & (1 << i)))
		{
			//sample the light target for the current subsample
			float4 lighting = lightTexture.Load(sampleIndices, i);

			//separate into diffuse and specular
			diffuse += lighting.xyz;
			specular += lighting.w;

			++numSamplesApplied;
		}
	}

	//apply a diffuse normalization factor
	const float diffuseNormalize = 1.0f / 3.14159265f;
	diffuse *= diffuseNormalize;

	//Apply the diffuse and specular albedo to the lighting values
	float3 diff = diffuse * diffuseAlbedo;
	float3 spec = specular * specularAlbedo;

	float3 output = (diff + spec) / numSamplesApplied;

	// calculate the RGBL value of the pixel
	output = sqrt(output);
	float luma = dot(output, float3(0.299, 0.587, 0.114));

	//final out is sum of diffuse and specular
	return float4(output, luma);
}