#include "../res/shaders/shader_settings.hlsli"
//Textures
Texture2DMS<float4, SAMPLE_COUNT> gBufferTexture : register(t0);
Texture2DMS<float4, SAMPLE_COUNT> positionTexture: register(t1);
Texture2D shadowMap : register(t2);

SamplerState ShadowMapSampler : register(s0);

//Constants
cbuffer LightParams : register(b0)
{
	float4 lightPos;
	float4 lightColor;
	float4 lightDirection;
	float4 lightRange;
	float2 spotlightAngles;
	float lightType;
	float shadowsOn;
};

cbuffer CameraParams : register(b1)
{
	float3 cameraPos;
	float padding2;
};

cbuffer MatrixParams : register(b2)
{
	matrix lightView;
	matrix lightProj;
};

#define AMBIENTLIGHT 0.0f
#define DIRECTIONALLIGHT 1.0f
#define POINTLIGHT 2.0f
#define SPOTLIGHT 3.0f

//function for decoding normals from the sphere map
float3 SpheremapDecode(float2 encoded)
{
	float4 nn = float4(encoded, 1, -1);
	float l = dot(nn.xyz, -nn.xyw);
	nn.z = l;
	nn.xy *= sqrt(l);
	return nn.xyz * 2 + float3(0, 0, -1);
}

void GetOcclusionData(in float2 screenPos, out float occlusion)
{
	//determine indices for sampling the texture from screen position
	int3 sampleIndices = int3((screenPos.xy / 2), 0);
	occlusion = shadowMap.Load(sampleIndices).x;
}

//function for extracting g-buffer attributes
void GetGBufferAttributes(in float2 screenPos, in int ssIndex, out float3 normal, out float3 position, out float specularPower, out float distance)
{
	//determine indices for sampling the texture from screen position
	int2 sampleIndices = int2(screenPos.xy);
	float2 texCoords = screenPos;
	float4 normalTex = gBufferTexture.Load(sampleIndices, ssIndex);
	float2 encodedNormal = normalTex.xy;
	normal = SpheremapDecode(encodedNormal);
	specularPower = normalTex.w;
	float4 positionData = positionTexture.Load(sampleIndices, ssIndex);
	position = positionData.xyz;
	distance = positionData.w;
}

//Calculate basic lighting for g-buffer texel
float4 CalcLighting(in float2 screenPos, in float3 normal, in float3 position, in float specularPower, in float distance)
{
	float2 projTexCoord;

	//calculate the diffuse term;
	float3 L = 0;
	float attenuation = 1.0f;
	
	float3 diffuse = float3(0.0f, 0.0f, 0.0f);
	float specular = 0.0f;

	// if ambient light simply output the ambient color with occlusion
	if (lightType == AMBIENTLIGHT)
	{
		float occlusion;
		GetOcclusionData(screenPos.xy, occlusion);

		diffuse = saturate(lightColor.xyz * occlusion);
		specular = 0.0f; // ambient lights probably don't need specularity
	}
	else
	{
		// Calculate the position of the pixel in light projection space
		float4 lightViewPosition = mul(float4(position.xyz, 1.0f), lightView);
		lightViewPosition = mul(lightViewPosition, lightProj);
		lightViewPosition = lightViewPosition / lightViewPosition.w;

		lightViewPosition.z = lightViewPosition.z - 0.001f;

		// Calculate shadow map projected tex coords
		projTexCoord.x = (lightViewPosition.x) / 2.0f + 0.5f;
		projTexCoord.y = (-lightViewPosition.y) / 2.0f + 0.5f;
		
		// check if the pixel is in view of the light
		if ((saturate(projTexCoord.x) == projTexCoord.x) && (saturate(projTexCoord.y) == projTexCoord.y) || shadowsOn == 0.0f)
		{
			// Sample the shadow map
			float depthValue = shadowMap.Sample(ShadowMapSampler, projTexCoord).r;

			if (lightViewPosition.z <= depthValue || shadowsOn == 0.0f)
			{
				if (lightType == POINTLIGHT || lightType == SPOTLIGHT)
				{
					//base the light vector on light position
					L = lightPos.xyz - position;
					//calculate attenuation based on distance from the light source
					float dist = length(L);
					attenuation = max(0, 1.0f - (dist / lightRange.x));

					L /= dist;

					L = normalize(L);
				}
				else if (lightType == DIRECTIONALLIGHT)
				{
					//light direction is explicit for directional lights
					L = -lightDirection.xyz;
				}

				if (lightType == SPOTLIGHT)
				{
					//Also add in the spotlight attenuation factor
					float3 L2 = lightDirection.xyz;
					float rho = dot(-L, L2);
					attenuation *= saturate((rho - spotlightAngles.y) / (spotlightAngles.x - spotlightAngles.y));
				}

				float nDotL = saturate(dot(normal, L));
				diffuse = nDotL * lightColor.xyz * attenuation;

				//Calculate the specular amount
				float3 V = cameraPos - position;
				float3 H;
				if (lightType == DIRECTIONALLIGHT)
				{
					H = normalize(L + V);
				}
				else
				{
					H = normalize(L + V);
				}

				specular = pow(saturate(dot(normal, H)), specularPower) * attenuation * nDotL;
			}
		}
	}

	//final value is diffuse rgb and mono specular
	return float4(diffuse, specular);
}

//Lighting pass pixel shader code
float4 main(in float4 screenPos : SV_POSITION) : SV_Target0
{
	float3 normal;
	float3 position;
	float specularPower;
	float distance;

	//get g-buffer values
	GetGBufferAttributes(screenPos.xy, 0, normal, position, specularPower, distance);

	return CalcLighting(screenPos.xy, normal, position, specularPower, distance);
}

// Lighting pass per sample code
float4 ssMain(in float4 screenPos : SV_POSITION, in uint subSampleIndex : SV_SampleIndex) : SV_Target0
{
	float3 normal;
	float3 position;
	float specularPower;
	float distance;

	//get g-buffer values
	GetGBufferAttributes(screenPos.xy, subSampleIndex, normal, position, specularPower, distance);

	return CalcLighting(screenPos.xy, normal, position, specularPower, distance);
}